# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_phonetic_edit_distance.

This module contains unit tests for abydos.distance.PhoneticEditDistance
"""

import unittest

from abydos.distance import PhoneticEditDistance


class PhoneticEditDistanceTestCases(unittest.TestCase):
    """Test phonetic edit distance functions.

    abydos.distance.PhoneticEditDistance
    """

    ped = PhoneticEditDistance()

    def test_phonetic_edit_distance_dist(self):
        """Test abydos.distance.PhoneticEditDistance.dist."""
        # Base cases
        self.assertEqual(self.ped.dist('', ''), 0.0)
        self.assertEqual(self.ped.dist('a', ''), 1.0)
        self.assertEqual(self.ped.dist('', 'a'), 1.0)
        self.assertEqual(self.ped.dist('abc', ''), 1.0)
        self.assertEqual(self.ped.dist('', 'abc'), 1.0)
        self.assertEqual(self.ped.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.ped.dist('abcd', 'efgh'), 0.10483870967741934)

        self.assertAlmostEqual(
            self.ped.dist('Nigel', 'Niall'), 0.1774193548387097
        )
        self.assertAlmostEqual(
            self.ped.dist('Niall', 'Nigel'), 0.1774193548387097
        )
        self.assertAlmostEqual(
            self.ped.dist('Colin', 'Coiln'), 0.1741935483870968
        )
        self.assertAlmostEqual(
            self.ped.dist('Coiln', 'Colin'), 0.1741935483870968
        )
        self.assertAlmostEqual(
            self.ped.dist('ATCAACGAGT', 'AACGATTAG'), 0.2370967741935484
        )

    def test_phonetic_edit_distance_dist_abs(self):
        """Test abydos.distance.PhoneticEditDistance.dist_abs."""
        # Base cases
        self.assertEqual(self.ped.dist_abs('', ''), 0)
        self.assertEqual(self.ped.dist_abs('a', ''), 1)
        self.assertEqual(self.ped.dist_abs('', 'a'), 1)
        self.assertEqual(self.ped.dist_abs('abc', ''), 3)
        self.assertEqual(self.ped.dist_abs('', 'abc'), 3)
        self.assertEqual(self.ped.dist_abs('abc', 'abc'), 0)
        self.assertEqual(self.ped.dist_abs('abcd', 'efgh'), 0.4193548387096774)

        self.assertAlmostEqual(
            self.ped.dist_abs('Nigel', 'Niall'), 0.8870967741935485
        )
        self.assertAlmostEqual(
            self.ped.dist_abs('Niall', 'Nigel'), 0.8870967741935485
        )
        self.assertAlmostEqual(
            self.ped.dist_abs('Colin', 'Coiln'), 0.870967741935484
        )
        self.assertAlmostEqual(
            self.ped.dist_abs('Coiln', 'Colin'), 0.870967741935484
        )
        self.assertAlmostEqual(
            self.ped.dist_abs('ATCAACGAGT', 'AACGATTAG'), 2.370967741935484
        )

        self.assertEqual(
            PhoneticEditDistance(weights={'syllabic': 1.0}).dist_abs(
                'Nigel', 'Niall'
            ),
            0.0,
        )
        self.assertAlmostEqual(
            PhoneticEditDistance(weights=(1, 1, 1)).dist_abs('Nigel', 'Niall'),
            0.33333333333333326,
        )
        self.assertAlmostEqual(
            PhoneticEditDistance(mode='osa').dist_abs('Niel', 'Neil'),
            0.06451612903225801,
        )

    def test_phonetic_edit_distance_alignment(self):
        """Test abydos.distance.PhoneticEditDistance.alignment."""
        # Base cases
        self.assertEqual(self.ped.alignment('', ''), (0.0, '', ''))
        self.assertEqual(self.ped.alignment('a', ''), (1.0, 'a', '-'))
        self.assertEqual(self.ped.alignment('', 'a'), (1.0, '-', 'a'))
        self.assertEqual(self.ped.alignment('abc', ''), (3.0, 'abc', '---'))
        self.assertEqual(self.ped.alignment('', 'abc'), (3.0, '---', 'abc'))
        self.assertEqual(self.ped.alignment('abc', 'abc'), (0.0, 'abc', 'abc'))
        self.assertEqual(
            self.ped.alignment('abcd', 'efgh'),
            (0.4193548387096774, 'abcd', 'efgh'),
        )

        self.assertEqual(
            self.ped.alignment('Nigel', 'Niall'),
            (0.8870967741935485, 'Nigel', 'Niall'),
        )
        self.assertEqual(
            self.ped.alignment('Niall', 'Nigel'),
            (0.8870967741935485, 'Niall', 'Nigel'),
        )
        self.assertEqual(
            self.ped.alignment('Colin', 'Coiln'),
            (0.870967741935484, 'Colin', 'Coiln'),
        )
        self.assertEqual(
            self.ped.alignment('Coiln', 'Colin'),
            (0.870967741935484, 'Coiln', 'Colin'),
        )
        self.assertEqual(
            PhoneticEditDistance(mode='osa').alignment('Niel', 'Neil'),
            (0.06451612903225801, 'Niel', 'Neil'),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_vps.

This module contains unit tests for abydos.distance.VPS
"""

import unittest

from abydos.distance import VPS


class VPSTestCases(unittest.TestCase):
    """Test VPS functions.

    abydos.distance.VPS
    """

    cmp = VPS()

    def test_vps_sim(self):
        """Test abydos.distance.VPS.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim('a', 'a'), 1.0)
        self.assertEqual(self.cmp.sim('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.0)

        # Testcases from paper
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'ANSCOMB'), 0.5972222222222222
        )
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'ANSCOMBE'), 0.7083333333333334
        )
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'BRANSCOMB'), 0.5879629629629629
        )
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'DASCOMBE'), 0.5925925925925926
        )
        self.assertEqual(self.cmp.sim('AINSCOMBE', 'DUNSCOMBE'), 0.75)
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'HANSCOMB'), 0.6620370370370371
        )
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'LIPSCOMBE'), 0.6666666666666666
        )
        self.assertEqual(
            self.cmp.sim('AINSCOMBE', 'LUSCOMBE'), 0.5555555555555556
        )
        self.assertEqual(self.cmp.sim('JULIA', 'JULIAN'), 0.8)
        self.assertEqual(self.cmp.sim('JULIA', 'JULIANA'), 0.7063492063492063)
        self.assertEqual(self.cmp.sim('JULIA', 'JULIANNA'), 0.6011904761904762)
        self.assertEqual(self.cmp.sim('JULIA', 'JULIE'), 0.75)
        self.assertEqual(self.cmp.sim('JULIA', 'JULIET'), 0.6)
        self.assertEqual(self.cmp.sim('JULIA', 'JULIUS'), 0.6333333333333333)
        self.assertEqual(self.cmp.sim('ROBBINS', 'ROBYNS'), 0.5238095238095238)


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_eudex.

This module contains unit tests for abydos.phonetic.Eudex
"""

import unittest

from abydos.phonetic import Eudex


class EudexTestCases(unittest.TestCase):
    """Test eudex functions.

    test cases for abydos.phonetic.Eudex
    """

    pa = Eudex()

    def test_eudex(self):
        """Test abydos.phonetic.Eudex."""
        # base cases
        self.assertEqual(self.pa.encode(''), '18374686479671623680')
        self.assertEqual(self.pa.encode(' '), '18374686479671623680')

        # exact & mismatch cases from
        # https://github.com/ticki/eudex/blob/master/src/tests.rs
        self.assertEqual(self.pa.encode('JAva'), self.pa.encode('jAva'))
        self.assertEqual(
            self.pa.encode('co!mputer'), self.pa.encode('computer')
        )
        self.assertEqual(
            self.pa.encode('comp-uter'), self.pa.encode('computer')
        )
        self.assertEqual(
            self.pa.encode('comp@u#te?r'), self.pa.encode('computer')
        )
        self.assertEqual(self.pa.encode('lal'), self.pa.encode('lel'))
        self.assertEqual(self.pa.encode('rindom'), self.pa.encode('ryndom'))
        self.assertEqual(
            self.pa.encode('riiiindom'), self.pa.encode('ryyyyyndom')
        )
        self.assertEqual(
            self.pa.encode('riyiyiiindom'), self.pa.encode('ryyyyyndom')
        )
        self.assertEqual(
            self.pa.encode('triggered'), self.pa.encode('TRIGGERED')
        )
        self.assertEqual(self.pa.encode('repert'), self.pa.encode('ropert'))

        self.assertNotEqual(self.pa.encode('reddit'), self.pa.encode('eddit'))
        self.assertNotEqual(self.pa.encode('lol'), self.pa.encode('lulz'))
        self.assertNotEqual(self.pa.encode('ijava'), self.pa.encode('java'))
        self.assertNotEqual(self.pa.encode('jiva'), self.pa.encode('java'))
        self.assertNotEqual(self.pa.encode('jesus'), self.pa.encode('iesus'))
        self.assertNotEqual(self.pa.encode('aesus'), self.pa.encode('iesus'))
        self.assertNotEqual(self.pa.encode('iesus'), self.pa.encode('yesus'))
        self.assertNotEqual(self.pa.encode('rupirt'), self.pa.encode('ropert'))
        self.assertNotEqual(self.pa.encode('ripert'), self.pa.encode('ropyrt'))
        self.assertNotEqual(self.pa.encode('rrr'), self.pa.encode('rraaaa'))
        self.assertNotEqual(
            self.pa.encode('randomal'), self.pa.encode('randomai')
        )

        # manually checked against algorithm
        self.assertEqual(self.pa.encode('guillaume'), '288230383131034112')
        self.assertEqual(self.pa.encode('niall'), '648518346341351840')
        self.assertEqual(self.pa.encode('hello'), '144115188075896832')
        self.assertEqual(self.pa.encode('christopher'), '433648490138894409')
        self.assertEqual(self.pa.encode('colin'), '432345564238053650')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._cosine.

Cosine similarity & distance
"""

from math import sqrt
from typing import Any, Optional

from ._token_distance import _TokenDistance
from ..tokenizer import _Tokenizer

__all__ = ['Cosine']


class Cosine(_TokenDistance):
    r"""Cosine similarity.

    For two sets X and Y, the cosine similarity, Otsuka-Ochiai coefficient, or
    Ochiai coefficient :cite:`Otsuka:1936,Ochiai:1957` is

        .. math::

            sim_{cosine}(X, Y) = \frac{|X \cap Y|}{\sqrt{|X| \cdot |Y|}}

    In :ref:`2x2 confusion table terms <confusion_table>`, where a+b+c+d=n,
    this is

        .. math::

            sim_{cosine} =
            \frac{a}{\sqrt{(a+b)(a+c)}}

    Notes
    -----
    This measure is also known as the Fowlkes-Mallows index
    :cite:`Fowlkes:1983` for two classes and G-measure, the geometric mean of
    precision & recall.


    .. versionadded:: 0.3.6

    """

    def __init__(
        self,
        tokenizer: Optional[_Tokenizer] = None,
        intersection_type: str = 'crisp',
        **kwargs: Any
    ) -> None:
        """Initialize Cosine instance.

        Parameters
        ----------
        tokenizer : _Tokenizer
            A tokenizer instance from the :py:mod:`abydos.tokenizer` package
        intersection_type : str
            Specifies the intersection type, and set type as a result:
            See :ref:`intersection_type <intersection_type>` description in
            :py:class:`_TokenDistance` for details.
        **kwargs
            Arbitrary keyword arguments

        Other Parameters
        ----------------
        qval : int
            The length of each q-gram. Using this parameter and tokenizer=None
            will cause the instance to use the QGram tokenizer with this
            q value.
        metric : _Distance
            A string distance measure class for use in the ``soft`` and
            ``fuzzy`` variants.
        threshold : float
            A threshold value, similarities above which are counted as
            members of the intersection for the ``fuzzy`` variant.


        .. versionadded:: 0.4.0

        """
        super(Cosine, self).__init__(
            tokenizer=tokenizer, intersection_type=intersection_type, **kwargs
        )

    def sim(self, src: str, tar: str) -> float:
        r"""Return the cosine similarity of two strings.

        Parameters
        ----------
        src : str
            Source string (or QGrams/Counter objects) for comparison
        tar : str
            Target string (or QGrams/Counter objects) for comparison

        Returns
        -------
        float
            Cosine similarity

        Examples
        --------
        >>> cmp = Cosine()
        >>> cmp.sim('cat', 'hat')
        0.5
        >>> cmp.sim('Niall', 'Neil')
        0.3651483716701107
        >>> cmp.sim('aluminum', 'Catalan')
        0.11785113019775793
        >>> cmp.sim('ATCG', 'TAGC')
        0.0


        .. versionadded:: 0.1.0
        .. versionchanged:: 0.3.6
            Encapsulated in class

        """
        if src == tar:
            return 1.0
        if not src or not tar:
            return 0.0

        self._tokenize(src, tar)

        num = self._intersection_card()

        if num:
            return num / sqrt(self._src_card() * self._tar_card())
        return 0.0


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._mra.

The Match Rating Algorithm's distance measure
"""

from ._distance import _Distance
from ..phonetic import MRA as MRAPhonetic  # noqa: N811

__all__ = ['MRA']


class MRA(_Distance):
    """Match Rating Algorithm comparison rating.

    The Western Airlines Surname Match Rating Algorithm comparison rating, as
    presented on page 18 of :cite:`Moore:1977`.

    .. versionadded:: 0.3.6
    """

    _phonetic_alg = MRAPhonetic()

    def dist_abs(self, src: str, tar: str) -> float:
        """Return the MRA comparison rating of two strings.

        Parameters
        ----------
        src : str
            Source string for comparison
        tar : str
            Target string for comparison

        Returns
        -------
        int
            MRA comparison rating

        Examples
        --------
        >>> cmp = MRA()
        >>> cmp.dist_abs('cat', 'hat')
        5
        >>> cmp.dist_abs('Niall', 'Neil')
        6
        >>> cmp.dist_abs('aluminum', 'Catalan')
        0
        >>> cmp.dist_abs('ATCG', 'TAGC')
        5


        .. versionadded:: 0.1.0
        .. versionchanged:: 0.3.6
            Encapsulated in class

        """
        if src == tar:
            return 6
        if src == '' or tar == '':
            return 0
        src_tok = list(self._phonetic_alg.encode(src))
        tar_tok = list(self._phonetic_alg.encode(tar))

        if abs(len(src_tok) - len(tar_tok)) > 2:
            return 0

        length_sum = len(src_tok) + len(tar_tok)
        if length_sum < 5:
            min_rating = 5
        elif length_sum < 8:
            min_rating = 4
        elif length_sum < 12:
            min_rating = 3
        else:
            min_rating = 2

        for _ in range(2):
            new_src = []
            new_tar = []
            minlen = min(len(src_tok), len(tar_tok))
            for i in range(minlen):
                if src_tok[i] != tar_tok[i]:
                    new_src.append(src_tok[i])
                    new_tar.append(tar_tok[i])
            src_tok = new_src + src_tok[minlen:]
            tar_tok = new_tar + tar_tok[minlen:]
            src_tok.reverse()
            tar_tok.reverse()

        similarity = 6 - max(len(src_tok), len(tar_tok))

        if similarity >= min_rating:
            return similarity
        return 0

    def sim(self, src: str, tar: str) -> float:
        """Return the normalized MRA similarity of two strings.

        This is the MRA normalized to :math:`[0, 1]`, given that MRA itself is
        constrained to the range :math:`[0, 6]`.

        Parameters
        ----------
        src : str
            Source string for comparison
        tar : str
            Target string for comparison

        Returns
        -------
        float
            Normalized MRA similarity

        Examples
        --------
        >>> cmp = MRA()
        >>> cmp.sim('cat', 'hat')
        0.8333333333333334
        >>> cmp.sim('Niall', 'Neil')
        1.0
        >>> cmp.sim('aluminum', 'Catalan')
        0.0
        >>> cmp.sim('ATCG', 'TAGC')
        0.8333333333333334


        .. versionadded:: 0.1.0
        .. versionchanged:: 0.3.6
            Encapsulated in class

        """
        return self.dist_abs(src, tar) / 6


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_s_stemmer.

This module contains unit tests for abydos.stemmer.SStemmer
"""

import unittest

from abydos.stemmer import SStemmer


class SStemmerTestCases(unittest.TestCase):
    """Test S-stemmer functions.

    abydos.stemmer.SStemmer
    """

    stmr = SStemmer()

    def test_s_stemmer(self):
        """Test abydos.stemmer.SStemmer."""
        # Base case
        self.assertEqual(self.stmr.stem(''), '')

        # Tests from Harman paper
        self.assertEqual(self.stmr.stem('panels'), 'panel')
        self.assertEqual(self.stmr.stem('subjected'), 'subjected')
        self.assertEqual(self.stmr.stem('aerodynamics'), 'aerodynamic')
        self.assertEqual(self.stmr.stem('heating'), 'heating')

        # Additional tests to complete coverage
        self.assertEqual(self.stmr.stem('dairies'), 'dairy')
        self.assertEqual(self.stmr.stem('census'), 'census')
        self.assertEqual(self.stmr.stem('boss'), 'boss')
        self.assertEqual(self.stmr.stem('bosses'), 'bosse')
        self.assertEqual(self.stmr.stem('raises'), 'raise')
        self.assertEqual(self.stmr.stem('fees'), 'fee')
        self.assertEqual(self.stmr.stem('attourneies'), 'attourneie')
        self.assertEqual(self.stmr.stem('portemonnaies'), 'portemonnaie')
        self.assertEqual(self.stmr.stem('foes'), 'foe')
        self.assertEqual(self.stmr.stem('sundaes'), 'sundae')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_phonix.

This module contains unit tests for abydos.phonetic.Phonix
"""

import unittest

from abydos.phonetic import Phonix


class PhonixTestCases(unittest.TestCase):
    """Test Phonix functions.

    test cases for abydos.phonetic.Phonix
    """

    pa = Phonix()

    def test_phonix(self):
        """Test abydos.phonetic.Phonix."""
        self.assertEqual(self.pa.encode(''), '0000')

        # http://cpansearch.perl.org/src/MAROS/Text-Phonetic-2.05/t/007_phonix.t
        self.assertEqual(self.pa.encode('Müller'), 'M400')
        self.assertEqual(self.pa.encode('schneider'), 'S530')
        self.assertEqual(self.pa.encode('fischer'), 'F800')
        self.assertEqual(self.pa.encode('weber'), 'W100')
        self.assertEqual(self.pa.encode('meyer'), 'M000')
        self.assertEqual(self.pa.encode('wagner'), 'W250')
        self.assertEqual(self.pa.encode('schulz'), 'S480')
        self.assertEqual(self.pa.encode('becker'), 'B200')
        self.assertEqual(self.pa.encode('hoffmann'), 'H755')
        self.assertEqual(self.pa.encode('schäfer'), 'S700')
        self.assertEqual(self.pa.encode('schmidt'), 'S530')

        # http://cpansearch.perl.org/src/MAROS/Text-Phonetic-2.05/t/007_phonix.t:
        # testcases from Wais Module
        self.assertEqual(self.pa.encode('computer'), 'K513')
        self.assertEqual(self.pa.encode('computers'), 'K513')
        self.assertEqual(Phonix(5).encode('computers'), 'K5138')
        self.assertEqual(self.pa.encode('pfeifer'), 'F700')
        self.assertEqual(self.pa.encode('pfeiffer'), 'F700')
        self.assertEqual(self.pa.encode('knight'), 'N300')
        self.assertEqual(self.pa.encode('night'), 'N300')

        # http://cpansearch.perl.org/src/MAROS/Text-Phonetic-2.05/t/007_phonix.t:
        # testcases from
        # http://www.cl.uni-heidelberg.de/~bormann/documents/phono/
        # They use a sliglty different algorithm (first char is not included in
        # num code here)
        self.assertEqual(self.pa.encode('wait'), 'W300')
        self.assertEqual(self.pa.encode('weight'), 'W300')
        self.assertEqual(self.pa.encode('gnome'), 'N500')
        self.assertEqual(self.pa.encode('noam'), 'N500')
        self.assertEqual(self.pa.encode('rees'), 'R800')
        self.assertEqual(self.pa.encode('reece'), 'R800')
        self.assertEqual(self.pa.encode('yaeger'), 'v200')

        # http://books.google.com/books?id=xtWPI7Is9wIC&lpg=PA29&ots=DXhaL7ZkvK&dq=phonix%20gadd&pg=PA29#v=onepage&q=phonix%20gadd&f=false
        self.assertEqual(self.pa.encode('alam'), 'v450')
        self.assertEqual(self.pa.encode('berkpakaian'), 'B212')
        self.assertEqual(self.pa.encode('capaian'), 'K150')

        # http://books.google.com/books?id=LZrT6eWf9NMC&lpg=PA76&ots=Tex3FqNwGP&dq=%22phonix%20algorithm%22&pg=PA75#v=onepage&q=%22phonix%20algorithm%22&f=false
        self.assertEqual(self.pa.encode('peter'), 'P300')
        self.assertEqual(self.pa.encode('pete'), 'P300')
        self.assertEqual(self.pa.encode('pedro'), 'P360')
        self.assertEqual(self.pa.encode('stephen'), 'S375')
        self.assertEqual(self.pa.encode('steve'), 'S370')
        self.assertEqual(self.pa.encode('smith'), 'S530')
        self.assertEqual(self.pa.encode('smythe'), 'S530')
        self.assertEqual(self.pa.encode('gail'), 'G400')
        self.assertEqual(self.pa.encode('gayle'), 'G400')
        self.assertEqual(self.pa.encode('christine'), 'K683')
        self.assertEqual(self.pa.encode('christina'), 'K683')
        self.assertEqual(self.pa.encode('kristina'), 'K683')

        # max_length bounds tests
        self.assertEqual(
            Phonix(max_length=-1).encode('Niall'), 'N4' + '0' * 62
        )
        self.assertEqual(Phonix(max_length=0).encode('Niall'), 'N400')

        # zero_pad tests
        self.assertEqual(
            Phonix(max_length=-1, zero_pad=False).encode('Niall'), 'N4'
        )
        self.assertEqual(
            Phonix(max_length=0, zero_pad=False).encode('Niall'), 'N4'
        )
        self.assertEqual(
            Phonix(max_length=0, zero_pad=True).encode('Niall'), 'N400'
        )
        self.assertEqual(Phonix(max_length=4, zero_pad=False).encode(''), '0')
        self.assertEqual(
            Phonix(max_length=4, zero_pad=True).encode(''), '0000'
        )

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Müller'), 'ML')
        self.assertEqual(self.pa.encode_alpha('schneider'), 'SNT')
        self.assertEqual(self.pa.encode_alpha('fischer'), 'FS')
        self.assertEqual(self.pa.encode_alpha('weber'), 'WP')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_guth.

This module contains unit tests for abydos.distance.Guth
"""

import unittest

from abydos.distance import Guth


class GuthTestCases(unittest.TestCase):
    """Test Guth functions.

    abydos.distance.Guth
    """

    cmp = Guth()

    def test_guth_sim_score(self):
        """Test abydos.distance.Guth.sim_score."""
        # Base cases
        self.assertEqual(self.cmp.sim_score('', ''), 1.0)
        self.assertEqual(self.cmp.sim_score('a', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim_score('a', 'a'), 1.0)
        self.assertEqual(self.cmp.sim_score('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim_score('abcd', 'efgh'), 0.0)

        # Testcases from paper
        self.assertEqual(self.cmp.sim_score('Glawyn', 'Glavin'), 1.0)
        self.assertEqual(self.cmp.sim_score('Smears', 'Smares'), 1.0)
        self.assertEqual(self.cmp.sim_score('Giddings', 'Gittins'), 1.0)
        self.assertEqual(self.cmp.sim_score('Bokenham', 'Buckingham'), 0.0)

        # coverage
        self.assertAlmostEqual(
            Guth(qval=2).sim_score('Giddings', 'Gittins'), 0.0
        )

    def test_guth_sim(self):
        """Test abydos.distance.Guth.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim('a', 'a'), 1.0)
        self.assertEqual(self.cmp.sim('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.0)

        # Testcases from paper
        self.assertAlmostEqual(self.cmp.sim('Glawyn', 'Glavin'), 0.8)
        self.assertAlmostEqual(self.cmp.sim('Smears', 'Smares'), 0.86666666666)
        self.assertAlmostEqual(self.cmp.sim('Giddings', 'Gittins'), 0.8)
        self.assertAlmostEqual(self.cmp.sim('Bokenham', 'Buckingham'), 0.65)

        # coverage
        self.assertAlmostEqual(Guth(qval=2).sim('Giddings', 'Gittins'), 0.6)
        self.assertAlmostEqual(self.cmp.sim('abcfefed', 'abcfed'), 0.7)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.compression.test_compression_rle.

This module contains unit tests for abydos.compression.RLE
"""

import unittest

from abydos.compression import BWT, RLE


class RLETestCases(unittest.TestCase):
    """Test abydos.compression.RLE.encode & .decode."""

    rle = RLE()
    bwt = BWT()

    bws = 'WWWWWWWWWWWWBWWWWWWWWWWWWBBBWWWWWWWWWWWWWWWWWWWWWWWWBWWWWWWWWWWWWWW'

    def test_rle_encode(self):
        """Test abydos.compression.RLE.encode."""
        self.assertEqual(self.rle.encode(''), '')
        self.assertEqual(self.rle.encode(self.bwt.encode('')), '\x00')
        self.assertEqual(self.rle.encode('banana'), 'banana')
        self.assertEqual(
            self.rle.encode(self.bwt.encode('banana')), 'annb\x00aa'
        )
        self.assertEqual(self.rle.encode(self.bws), '12WB12W3B24WB14W')
        self.assertEqual(
            self.rle.encode(self.bwt.encode(self.bws)), 'WWBWWB45WB\x003WB10WB'
        )
        self.assertEqual(self.rle.encode('Schifffahrt'), 'Schi3fahrt')

    def test_rle_decode(self):
        """Test abydos.compression.RLE.decode."""
        self.assertEqual(self.rle.decode(''), '')
        self.assertEqual(self.bwt.decode(self.rle.decode('\x00')), '')
        self.assertEqual(self.rle.decode('banana'), 'banana')
        self.assertEqual(
            self.bwt.decode(self.rle.decode('annb\x00aa')), 'banana'
        )
        self.assertEqual(self.rle.decode('12WB12W3B24WB14W'), self.bws)
        self.assertEqual(self.rle.decode('12W1B12W3B24W1B14W'), self.bws)
        self.assertEqual(
            self.bwt.decode(self.rle.decode('WWBWWB45WB\x003WB10WB')), self.bws
        )
        self.assertEqual(self.rle.decode('Schi3fahrt'), 'Schifffahrt')

    def test_rle_roundtripping(self):
        """Test abydos.compression.RLE.encode & .decode roundtripping."""
        self.assertEqual(self.rle.decode(self.rle.encode('')), '')
        self.assertEqual(
            self.bwt.decode(
                self.rle.decode(self.rle.encode(self.bwt.encode('')))
            ),
            '',
        )
        self.assertEqual(self.rle.decode(self.rle.encode('banana')), 'banana')
        self.assertEqual(
            self.bwt.decode(
                self.rle.decode(self.rle.encode(self.bwt.encode('banana')))
            ),
            'banana',
        )
        self.assertEqual(self.rle.decode(self.rle.encode(self.bws)), self.bws)
        self.assertEqual(
            self.bwt.decode(
                self.rle.decode(self.rle.encode(self.bwt.encode(self.bws)))
            ),
            self.bws,
        )
        self.assertEqual(
            self.rle.decode(self.rle.encode('Schifffahrt')), 'Schifffahrt'
        )
        self.assertEqual(
            self.bwt.decode(
                self.rle.decode(
                    self.rle.encode(self.bwt.encode('Schifffahrt'))
                )
            ),
            'Schifffahrt',
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_synoname.

This module contains unit tests for abydos.distance.Synoname
"""

import unittest

from abydos.distance import Synoname


class SynonameTestCases(unittest.TestCase):
    """Test Synoname functions.

    abydos.distance.Synoname
    """

    cmp = Synoname()

    def test_synoname_strip_punct(self):
        """Test abydos.distance.Synoname._synoname_strip_punct."""
        # Base case
        self.assertEqual(self.cmp._synoname_strip_punct(''), '')  # noqa: SF01

        self.assertEqual(
            self.cmp._synoname_strip_punct('abcdefg'), 'abcdefg'  # noqa: SF01
        )
        self.assertEqual(
            self.cmp._synoname_strip_punct("a'b-c,d!e:f%g"),  # noqa: SF01
            'abcdefg',
        )

    def test_synoname_word_approximation(self):
        """Test abydos.distance.Synoname._synoname_word_approximation."""
        # Base case
        self.assertEqual(
            self.cmp._synoname_word_approximation('', ''), 0  # noqa: SF01
        )

        self.assertEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'di Domenico di Bonaventura',
                'di Tomme di Nuto',
                'Cosimo',
                'Luca',
            ),
            0.4,
        )
        self.assertEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'Antonello da Messina',
                'Messina',
                '',
                'Antonello da',
                {
                    'gen_conflict': False,
                    'roman_conflict': False,
                    'src_specials': [(35, 'b'), (35, 'c')],
                    'tar_specials': [(35, 'b'), (35, 'c')],
                },
            ),
            0,
        )
        self.assertEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'louis ii',
                'louis ii',
                'sr jean',
                'sr  pierre',
                {
                    'gen_conflict': False,
                    'roman_conflict': False,
                    'src_specials': [(49, 'b'), (68, 'd'), (121, 'b')],
                    'tar_specials': [(49, 'b'), (68, 'd'), (121, 'b')],
                },
            ),
            0,
        )
        self.assertEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'louis ii',
                'louis ii',
                'il giovane',
                'sr cadet',
                {
                    'gen_conflict': False,
                    'roman_conflict': False,
                    'src_specials': [
                        (46, 'a'),
                        (49, 'b'),
                        (52, 'a'),
                        (68, 'd'),
                    ],
                    'tar_specials': [
                        (8, 'a'),
                        (49, 'b'),
                        (68, 'd'),
                        (121, 'a'),
                    ],
                },
            ),
            1,
        )
        self.assertAlmostEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'louis ii',
                'louis ii',
                'ste.-geo ste.',
                'ste.-jo ste.',
                {
                    'gen_conflict': False,
                    'roman_conflict': False,
                    'src_specials': [
                        (49, 'b'),
                        (68, 'd'),
                        (127, 'b'),
                        (127, 'X'),
                    ],
                    'tar_specials': [
                        (49, 'b'),
                        (68, 'd'),
                        (127, 'b'),
                        (127, 'X'),
                    ],
                },
            ),
            2 / 3,
        )
        self.assertAlmostEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'louis ii',
                'louis',
                'ste.-geo ste.',
                '',
                {
                    'gen_conflict': False,
                    'roman_conflict': False,
                    'src_specials': [
                        (49, 'b'),
                        (68, 'd'),
                        (127, 'b'),
                        (127, 'X'),
                    ],
                    'tar_specials': [],
                },
            ),
            0,
        )
        self.assertAlmostEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'lou ii', 'louis', 'louis iv', 'ste.', {}
            ),
            0,
        )
        self.assertEqual(
            self.cmp._synoname_word_approximation(  # noqa: SF01
                'ren',
                'loren ste.',
                '',
                '',
                {
                    'tar_specials': [(68, 'd'), (127, 'X')],
                    'src_specials': [(0, '')],
                },
            ),
            1,
        )

    def test_synoname_sim_type(self):
        """Test abydos.distance.Synoname.sim_type."""
        self.assertEqual(self.cmp.sim_type('', ''), 1)
        self.assertEqual(Synoname(tests=['exact']).sim_type('', ''), 1)
        self.assertEqual(Synoname(tests=[]).sim_type('', ''), 13)
        self.assertEqual(
            Synoname(tests=['nonsense-test']).sim_type('', ''), 13
        )
        self.assertEqual(Synoname(ret_name=True).sim_type('', ''), 'exact')

        # Types 1-12
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Pieter', ''), ('Brueghel', 'Pieter', '')
            ),
            'exact',
        )

        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel II', 'Pieter', ''), ('Brueghel I', 'Pieter', '')
            ),
            'no_match',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Breghel', 'Pieter', ''), ('Brueghel', 'Pieter', '')
            ),
            'omission',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Pieter', ''), ('Breghel', 'Pieter', '')
            ),
            'omission',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Piter', ''), ('Brueghel', 'Pieter', '')
            ),
            'omission',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Pieter', ''), ('Brueghel', 'Piter', '')
            ),
            'omission',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brughel', 'Pieter', ''), ('Breghel', 'Pieter', '')
            ),
            'substitution',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Breughel', 'Peter', ''), ('Breughel', 'Piter', '')
            ),
            'substitution',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Pieter', ''), ('Breughel', 'Pieter', '')
            ),
            'transposition',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Peiter', ''), ('Brueghel', 'Pieter', '')
            ),
            'transposition',
        )

        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel:', 'Pieter', ''), ('Brueghel', 'Pi-eter', '')
            ),
            'punctuation',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel,', 'Pieter', ''), ('Brueghel', 'Pieter...', '')
            ),
            'punctuation',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Seu rat', 'George Pierre', ''),
                ('Seu-rat', 'George-Pierre', ''),
            ),
            'punctuation',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Picasso', '', ''), ('Picasso', 'Pablo', '')
            ),
            'no_first',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Pereira', 'I. R.', ''), ('Pereira', 'Irene Rice', '')
            ),
            'initials',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Pereira', 'I.', ''), ('Pereira', 'Irene Rice', '')
            ),
            'initials',
        )
        self.assertNotEqual(
            Synoname(ret_name=True).sim_type(
                ('Pereira', 'I. R.', ''), ('Pereira', 'I. Smith', '')
            ),
            'initials',
        )
        self.assertNotEqual(
            Synoname(ret_name=True).sim_type(
                ('Pereira', 'I. R. S.', ''), ('Pereira', 'I. S. R.', '')
            ),
            'initials',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('de Goya', 'Francisco', ''),
                ('de Goya y Lucientes', 'Francisco', ''),
            ),
            'extension',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Seurat', 'George', ''), ('Seurat', 'George-Pierre', '')
            ),
            'extension',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Gericault', 'Theodore', ''),
                ('Gericault', 'Jean Louis Andre Theodore', ''),
            ),
            'inclusion',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Dore', 'Gustave', ''),
                ('Dore', 'Paul Gustave Louis Christophe', ''),
            ),
            'inclusion',
        )

        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Rosetti', 'Dante Gabriel', ''),
                ('Rosetti', 'Gabriel Charles Dante', ''),
            ),
            'word_approx',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('di Domenico di Bonaventura', 'Cosimo', ''),
                ('di Tomme di Nuto', 'Luca', ''),
            ),
            'no_match',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Pereira', 'I. R.', ''), ('Pereira', 'I. Smith', '')
            ),
            'word_approx',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Antonello da Messina', '', ''),
                ('Messina', 'Antonello da', ''),
            ),
            'confusions',
        )
        self.assertEqual(
            Synoname(ret_name=True).sim_type(
                ('Brueghel', 'Pietter', ''), ('Bruegghel', 'Pieter', '')
            ),
            'char_approx',
        )

    def test_synoname_dist_abs(self):
        """Test abydos.distance.Synoname.dist_abs."""
        # Base cases
        self.assertEqual(self.cmp.dist_abs('', ''), 1)
        self.assertEqual(Synoname(tests=['exact']).dist_abs('', ''), 1)
        self.assertEqual(Synoname(tests=[]).dist_abs('', ''), 13)
        self.assertEqual(
            Synoname(tests=['nonsense-test']).dist_abs('', ''), 13
        )

        # Test input formats
        self.assertEqual(
            self.cmp.dist_abs(
                ('Brueghel II (the Younger)', 'Pieter', 'Workshop of'),
                ('Brueghel II (the Younger)', 'Pieter', 'Workshop of'),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                'Brueghel II (the Younger)#Pieter#' + 'Workshop of',
                'Brueghel II (the Younger)#Pieter#' + 'Workshop of',
            ),
            1,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                '22#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
                '44#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
            ),
            1,
        )

        # approx_c tests
        self.assertEqual(
            self.cmp.dist_abs(
                (
                    'Master of Brueghel II (the Younger)',
                    'Pieter',
                    'Workshop of',
                ),
                ('Brueghel I (the Elder)', 'Pieter', 'Workshop of'),
            ),
            13,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                ('Master of Brueghel II', 'Pieter', 'Workshop of'),
                ('Master known as the Brueghel II', 'Pieter', 'Workshop of'),
            ),
            10,
        )

    def test_synoname_dist(self):
        """Test abydos.distance.Synoname.dist."""
        # Base cases
        self.assertAlmostEqual(self.cmp.dist('', ''), 1 / 14)
        self.assertAlmostEqual(
            self.cmp.dist(
                '22#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
                '44#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
            ),
            1 / 14,
        )
        self.assertAlmostEqual(
            self.cmp.dist(
                (
                    'Master of Brueghel II (the Younger)',
                    'Pieter',
                    'Workshop of',
                ),
                ('Brueghel I (the Elder)', 'Pieter', 'Workshop of'),
            ),
            13 / 14,
        )
        self.assertAlmostEqual(
            self.cmp.dist(
                ('Master of Brueghel II', 'Pieter', 'Workshop of'),
                ('Master known as the Brueghel II', 'Pieter', 'Workshop of'),
            ),
            10 / 14,
        )

    def test_synoname_sim(self):
        """Test abydos.distance.Synoname.sim."""
        # Base cases
        self.assertAlmostEqual(self.cmp.sim('', ''), 13 / 14)
        self.assertAlmostEqual(
            self.cmp.sim(
                '22#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
                '44#Brueghel II (the Younger)#Pieter#' + 'Workshop of',
            ),
            13 / 14,
        )
        self.assertAlmostEqual(
            self.cmp.sim(
                (
                    'Master of Brueghel II (the Younger)',
                    'Pieter',
                    'Workshop of',
                ),
                ('Brueghel I (the Elder)', 'Pieter', 'Workshop of'),
            ),
            1 / 14,
        )
        self.assertAlmostEqual(
            self.cmp.sim(
                ('Master of Brueghel II', 'Pieter', 'Workshop of'),
                ('Master known as the Brueghel II', 'Pieter', 'Workshop of'),
            ),
            4 / 14,
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_paice_husk.

This module contains unit tests for abydos.stemmer.PaiceHusk
"""

import unittest

from abydos.stemmer import PaiceHusk

from .. import _corpus_file


class PaiceHuskTestCases(unittest.TestCase):
    """Test Paice-Husk functions.

    abydos.stemmer.PaiceHusk
    """

    stmr = PaiceHusk()

    def test_paice_husk(self):
        """Test abydos.stemmer.PaiceHusk."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        # cases copied from
        # https://doi.org/10.1145/101306.101310
        self.assertEqual(self.stmr.stem('maximum'), 'maxim')
        self.assertEqual(self.stmr.stem('presumably'), 'presum')
        self.assertEqual(self.stmr.stem('multiply'), 'multiply')
        self.assertEqual(self.stmr.stem('provision'), 'provid')
        self.assertEqual(self.stmr.stem('owed'), 'ow')
        self.assertEqual(self.stmr.stem('owing'), 'ow')
        self.assertEqual(self.stmr.stem('ear'), 'ear')
        self.assertEqual(self.stmr.stem('saying'), 'say')
        self.assertEqual(self.stmr.stem('crying'), 'cry')
        self.assertEqual(self.stmr.stem('string'), 'string')
        self.assertEqual(self.stmr.stem('meant'), 'meant')
        self.assertEqual(self.stmr.stem('cement'), 'cem')

    def test_paice_husk_hopper_set(self):
        """Test abydos.stemmer.PaiceHusk (Hopper262 testset).

        Source:
        https://raw.githubusercontent.com/Hopper262/paice-husk-stemmer/master/wordlist.txt

        The only correction made from stemmed values in the Hopper262 set/
        implementations were:
         - ymca : ymc -> ymca
         - yttrium : yttr -> yttri
         - ywca : ywc -> ywca
        The Pascal reference implementation does not consider 'y' in initial
        position to be a vowel.
        """
        with open(_corpus_file('paicehusk.csv')) as hopper_ts:
            for hopper_line in hopper_ts:
                (word, stem) = hopper_line.strip().split(',')
                self.assertEqual(self.stmr.stem(word), stem)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_spfc.

This module contains unit tests for abydos.phonetic.SPFC
"""

import unittest

from abydos.phonetic import SPFC


class SPFCTestCases(unittest.TestCase):
    """Test SPFC functions.

    test cases for abydos.phonetic.SPFC
    """

    pa = SPFC()

    def test_spfc(self):
        """Test abydos.phonetic.SPFC."""
        self.assertEqual(self.pa.encode(''), '')

        # https://archive.org/stream/accessingindivid00moor#page/19/mode/1up
        self.assertEqual(self.pa.encode(('J', 'KUHNS')), '16760')
        self.assertEqual(self.pa.encode(('G', 'ALTSHULER')), '35797')
        self.assertEqual(self.pa.encode('J KUHNS'), '16760')
        self.assertEqual(self.pa.encode('G ALTSHULER'), '35797')
        self.assertEqual(self.pa.encode('J. KUHNS'), '16760')
        self.assertEqual(self.pa.encode('G. ALTSHULER'), '35797')
        self.assertEqual(self.pa.encode('J. Kuhns'), '16760')
        self.assertEqual(self.pa.encode('G. Altshuler'), '35797')
        self.assertEqual(self.pa.encode('T. Vines'), '16760')
        self.assertEqual(self.pa.encode('J. Butler'), '35779')
        self.assertNotEqual(
            self.pa.encode('J. Kuhns'), self.pa.encode('J. Kuntz')
        )
        self.assertEqual(self.pa.encode('Jon Kuhns'), '16760')
        self.assertEqual(self.pa.encode('James Kuhns'), '16760')

        self.assertRaises(AttributeError, self.pa.encode, ('J', 'A', 'Kuhns'))
        self.assertRaises(AttributeError, self.pa.encode, 'JKuhns')
        self.assertRaises(AttributeError, self.pa.encode, 5)

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('James Goldstein'), '77795')
        self.assertEqual(self.pa.encode('James Hansen'), '57760')
        self.assertEqual(self.pa.encode('James Hester'), '57700')
        self.assertEqual(self.pa.encode('James Bardot'), '31745')
        self.assertEqual(self.pa.encode('James Windsor'), '27765')
        self.assertEqual(self.pa.encode('James Wenders'), '27760')
        self.assertEqual(self.pa.encode('James Ventor'), '17760')
        self.assertEqual(self.pa.encode('þ þ'), '00')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('J. Kuhns'), 'CSGMS')
        self.assertEqual(self.pa.encode_alpha('G. Altshuler'), 'ARGEG')
        self.assertEqual(self.pa.encode_alpha('T. Vines'), 'CSGMS')
        self.assertEqual(self.pa.encode_alpha('James Ventor'), 'CZGMS')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.


"""abydos.tests.

This module contains unit tests for Abydos
"""

import os
import unittest
from random import random

CORPORA = os.path.join(os.path.dirname(__file__), 'corpora')

EXTREME_TEST = False  # Set to True to test EVERY single case (NB: takes hours)
ALLOW_RANDOM = True  # Set to False to skip all random tests

if not EXTREME_TEST and os.path.isfile(
    os.path.join(os.path.dirname(__file__), 'EXTREME_TEST')
):
    # EXTREME_TEST file detected -- switching to EXTREME_TEST mode...
    EXTREME_TEST = True

NIALL = (
    'Niall',
    'Neal',
    'Neil',
    'Njall',
    'Njáll',
    'Nigel',
    'Neel',
    'Nele',
    'Nigelli',
    'Nel',
    'Kneale',
    'Uí Néill',
    "O'Neill",
    'MacNeil',
    'MacNele',
    'Niall Noígíallach',
)

COLIN = (
    'Colin',
    'Collin',
    'Cullen',
    'Cuilen',
    'Cailean',
    'MacCailean',
    'Cuilén',
    'Colle',
    'Calum',
    'Callum',
    'Colinn',
    'Colon',
    'Colynn',
    'Col',
    'Cole',
    'Nicolas',
    'Nicholas',
    'Cailean Mór Caimbeul',
)

NONQ_FROM = 'The quick brown fox jumped over the lazy dog.'
NONQ_TO = 'That brown dog jumped over the fox.'


def _corpus_file(name, corpora_dir=CORPORA):
    """Return the path to a corpus file.

    Parameters
    ----------
    name : str
        Corpus file
        corpora_dir : str
        The directory containing the corpora

    Returns
    -------
    str: The full path to the corpus file

    """
    return os.path.join(corpora_dir, name)


def _one_in(inverse_probability):
    """Return whether to run a test.

    Return True if:
        EXTREME_TEST is True
        OR
        (ALLOW_RANDOM is True
        AND
        random.random() * inverse_probability < 1)
    Otherwise return False

    Parameters
    ----------
    inverse_probability : int
        The inverse of the probability

    Returns
    -------
    bool: Whether to run a test

    """
    if EXTREME_TEST:
        return True
    elif ALLOW_RANDOM and random() * inverse_probability < 1:  # noqa: S311
        return True
    else:
        return False


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint__fingerprint.

This module contains unit tests for abydos.fingerprint.Count
"""

import unittest

# noinspection PyProtectedMember
from abydos.fingerprint._fingerprint import _Fingerprint


class CountFingerprintTestCases(unittest.TestCase):
    """Test _Fingerprint class.

    abydos.fingerprint.Count
    """

    fp = _Fingerprint()

    def test_fingerprint_fingerprint(self):
        """Test abydos.fingerprint._Fingerprint.fingerprint."""
        # Base cases
        self.assertEqual(self.fp.fingerprint(''), '')
        self.assertEqual(self.fp.fingerprint('word'), 'word')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._quantitative_dice.

Quantitative Dice similarity
"""

from typing import Any, Optional

from ._token_distance import _TokenDistance
from ..tokenizer import _Tokenizer

__all__ = ['QuantitativeDice']


class QuantitativeDice(_TokenDistance):
    r"""Quantitative Dice similarity.

    For two multisets X and Y drawn from an alphabet S, Quantitative Dice
    similarity is

        .. math::

            sim_{QuantitativeDice}(X, Y) =
            \frac{2 \cdot \sum_{i \in S} X_iY_i}
            {\sum_{i \in S} X_i^2 + \sum_{i \in S} Y_i^2}

    .. versionadded:: 0.4.0
    """

    def __init__(
        self, tokenizer: Optional[_Tokenizer] = None, **kwargs: Any
    ) -> None:
        """Initialize QuantitativeDice instance.

        Parameters
        ----------
        tokenizer : _Tokenizer
            A tokenizer instance from the :py:mod:`abydos.tokenizer` package
        **kwargs
            Arbitrary keyword arguments

        Other Parameters
        ----------------
        qval : int
            The length of each q-gram. Using this parameter and tokenizer=None
            will cause the instance to use the QGram tokenizer with this
            q value.


        .. versionadded:: 0.4.0

        """
        super(QuantitativeDice, self).__init__(tokenizer=tokenizer, **kwargs)

    def sim(self, src: str, tar: str) -> float:
        """Return the Quantitative Dice similarity of two strings.

        Parameters
        ----------
        src : str
            Source string (or QGrams/Counter objects) for comparison
        tar : str
            Target string (or QGrams/Counter objects) for comparison

        Returns
        -------
        float
            Quantitative Dice similarity

        Examples
        --------
        >>> cmp = QuantitativeDice()
        >>> cmp.sim('cat', 'hat')
        0.5
        >>> cmp.sim('Niall', 'Neil')
        0.36363636363636365
        >>> cmp.sim('aluminum', 'Catalan')
        0.10526315789473684
        >>> cmp.sim('ATCG', 'TAGC')
        0.0


        .. versionadded:: 0.4.0

        """
        if src == tar:
            return 1.0

        self._tokenize(src, tar)

        alphabet = self._total().keys()

        return (
            2
            * sum(
                self._src_tokens[tok] * self._tar_tokens[tok]
                for tok in alphabet
            )
            / (
                sum(
                    self._src_tokens[tok] * self._src_tokens[tok]
                    for tok in alphabet
                )
                + sum(
                    self._tar_tokens[tok] * self._tar_tokens[tok]
                    for tok in alphabet
                )
            )
        )


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_sound_d.

This module contains unit tests for abydos.phonetic.SoundD
"""

import unittest

from abydos.phonetic import SoundD


class SoundDTestCases(unittest.TestCase):
    """Test class SoundD functions.

    test cases for abydos.phonetic.SoundD
    """

    pa = SoundD()

    def test_sound_d(self):
        """Test abydos.phonetic.SoundD."""
        # Base cases
        self.assertEqual(self.pa.encode(''), '0000')
        self.assertEqual(SoundD(max_length=6).encode(''), '000000')

        self.assertEqual(self.pa.encode('knight'), '5300')
        self.assertEqual(self.pa.encode('accept'), '2130')
        self.assertEqual(self.pa.encode('pneuma'), '5500')
        self.assertEqual(self.pa.encode('ax'), '2000')
        self.assertEqual(self.pa.encode('wherever'), '6160')
        self.assertEqual(self.pa.encode('pox'), '1200')
        self.assertEqual(self.pa.encode('anywhere'), '5600')
        self.assertEqual(self.pa.encode('adenosine'), '3525')
        self.assertEqual(self.pa.encode('judge'), '2200')
        self.assertEqual(self.pa.encode('rough'), '6000')
        self.assertEqual(self.pa.encode('x-ray'), '2600')
        self.assertEqual(
            SoundD(max_length=-1).encode('acetylcholine'), '234245'
        )
        self.assertEqual(SoundD(max_length=-1).encode('rough'), '6')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('pox'), 'PK')
        self.assertEqual(self.pa.encode_alpha('anywhere'), 'NR')
        self.assertEqual(self.pa.encode_alpha('adenosine'), 'TNKN')
        self.assertEqual(self.pa.encode_alpha('judge'), 'KK')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_henderson_heron.

This module contains unit tests for abydos.distance.HendersonHeron
"""

import unittest

from abydos.distance import HendersonHeron


class HendersonHeronTestCases(unittest.TestCase):
    """Test HendersonHeron functions.

    abydos.distance.HendersonHeron
    """

    cmp = HendersonHeron()

    def test_henderson_heron_dist(self):
        """Test abydos.distance.HendersonHeron.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 1.0)
        self.assertEqual(self.cmp.dist('a', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'a'), 1.0)
        self.assertAlmostEqual(self.cmp.dist('a', 'a'), 3.258008184e-06)
        self.assertEqual(self.cmp.dist('abc', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'abc'), 1.0)
        self.assertAlmostEqual(self.cmp.dist('abc', 'abc'), 6.40140979487e-11)
        self.assertEqual(self.cmp.dist('abcd', 'efgh'), 0.9684367974410505)

        self.assertAlmostEqual(self.cmp.dist('Nigel', 'Niall'), 4.94203602e-06)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Nigel'), 4.94203602e-06)
        self.assertAlmostEqual(self.cmp.dist('Colin', 'Coiln'), 4.94203602e-06)
        self.assertAlmostEqual(self.cmp.dist('Coiln', 'Colin'), 4.94203602e-06)
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 1.108779488e-12
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_sfinxbis.

This module contains unit tests for abydos.phonetic.SfinxBis
"""

import unittest

from abydos.phonetic import SfinxBis


class SfinxBisTestCases(unittest.TestCase):
    """Test SfinxBis functions.

    test cases for abydos.phonetic.SfinxBis
    """

    pa = SfinxBis()
    pa4 = SfinxBis(4)

    def test_sfinxbis(self):
        """Test abydos.phonetic.SfinxBis."""
        self.assertEqual(self.pa.encode(''), '')

        # http://www.swami.se/download/18.248ad5af12aa81365338000106/TestSfinx.txt
        # cases where the gold standard gave clearly wrong values have been
        # corrected below (marked with '# wrong'
        self.assertEqual(self.pa.encode('af Sandeberg'), 'S53162')
        self.assertEqual(self.pa.encode('av Ekenstam'), '$25835')
        self.assertEqual(self.pa.encode('Da Costa'), 'K83')
        self.assertEqual(self.pa.encode('Das Neves'), 'D8,N78')
        self.assertEqual(self.pa.encode('de Besche'), 'B8')
        self.assertEqual(self.pa.encode('de la Motte'), 'M3')
        self.assertEqual(self.pa.encode('de Las Heras'), 'H68')  # wrong
        self.assertEqual(self.pa.encode('de Los Santos'), 'S538')
        self.assertEqual(self.pa.encode('del Rosario'), 'R862')
        self.assertEqual(self.pa.encode('Den Boer'), 'B6')
        self.assertEqual(
            self.pa.encode('Der de Kazinczy'), 'D6,K8528'
        )  # wrong
        self.assertEqual(self.pa.encode('des Rieux'), 'R28')
        self.assertEqual(self.pa.encode('Di Luca'), 'L2')
        self.assertEqual(self.pa.encode('Do Rosario'), 'R862')
        self.assertEqual(self.pa.encode('Don Lind'), 'L53')
        self.assertEqual(self.pa.encode('Dos Santos'), 'S538')
        self.assertEqual(self.pa.encode('du Rietz'), 'R38')
        self.assertEqual(self.pa.encode('in de Betou'), 'B3')
        self.assertEqual(self.pa.encode('La Fleur'), 'F46')
        self.assertEqual(self.pa.encode('Le Grand'), 'G653')
        self.assertEqual(self.pa.encode('li Puma'), 'L,P5')
        self.assertEqual(self.pa.encode('lo Martire'), 'L,M636')
        self.assertEqual(self.pa.encode('mac Donald'), 'D543')
        self.assertEqual(self.pa.encode('mc Intosh'), '$538')
        self.assertEqual(self.pa.encode('S:t Cyr'), 'S6')
        self.assertEqual(self.pa.encode('Van Doom'), 'D5')
        self.assertEqual(self.pa.encode('Van de Peppel'), 'P14')
        self.assertEqual(self.pa.encode('Van den Berg'), 'B62')
        self.assertEqual(self.pa.encode('Van Der Kwast'), 'K783')
        self.assertEqual(self.pa.encode('von Ahn'), '$5')
        self.assertEqual(self.pa.encode('von Dem Knesebeck'), 'K5812')
        self.assertEqual(self.pa.encode('von Der Burg'), 'B62')
        self.assertEqual(self.pa.encode("D'Angelo"), 'D524')
        self.assertEqual(self.pa.encode("O'Conner"), '$256')
        self.assertEqual(self.pa.encode('Los'), 'L8')
        self.assertEqual(self.pa.encode('Mac'), 'M2')
        self.assertEqual(self.pa.encode('Till'), 'T4')
        self.assertEqual(self.pa.encode('Van'), 'V5')
        self.assertEqual(self.pa.encode('Von'), 'V5')
        self.assertEqual(
            self.pa.encode('Bernadotte af Wisborg'), 'B6533,V8162'
        )
        self.assertEqual(self.pa.encode('Hjort af Ornäs'), 'J63,$658')
        self.assertEqual(self.pa.encode('Horn af Åminne'), 'H65,$55')
        self.assertEqual(self.pa.encode('Horn av Åminne'), 'H65,$55')
        self.assertEqual(self.pa.encode('Hård af Segerstad'), 'H63,S26833')
        self.assertEqual(self.pa.encode('Hård av Segerstad'), 'H63,S26833')
        self.assertEqual(self.pa.encode('Stael von Holstein'), 'S34,H48325')
        self.assertEqual(self.pa.encode('de Oliveira e Silva'), '$4726,S47')
        self.assertEqual(self.pa.encode('de Alfaro y Gómez'), '$476,G58')
        self.assertEqual(self.pa.encode('Arjaliès-de la Lande'), '$6248,L53')
        self.assertEqual(
            self.pa.encode('Dominicus van den Bussche'), 'D5528,B8'
        )
        self.assertEqual(
            self.pa.encode('Edebol Eeg-Olofsson'), '$314,$2,$4785'
        )
        self.assertEqual(self.pa.encode('Jonsson-Blomqvist'), 'J585,B452783')
        self.assertEqual(self.pa.encode('Kiviniemi Birgersson'), '#755,B62685')
        self.assertEqual(
            self.pa.encode('Massena Serpa dos Santos'), 'M85,S61,S538'
        )
        self.assertEqual(self.pa.encode('S:t Clair Renard'), 'K426,R563')
        self.assertEqual(self.pa.encode('Skoog H Andersson'), 'S22,H,$53685')
        self.assertEqual(self.pa.encode('von Post-Skagegård'), 'P83,S22263')
        self.assertEqual(self.pa.encode('von Zur-Mühlen'), 'S6,M45')
        self.assertEqual(self.pa.encode('Waltå O:son'), 'V43,$85')
        self.assertEqual(
            self.pa.encode('Zardán Gómez de la Torre'), 'S635,G58,T6'
        )
        self.assertEqual(self.pa.encode('af Jochnick'), 'J252')
        self.assertEqual(self.pa.encode('af Ioscnick'), 'J8252')
        self.assertEqual(self.pa.encode('Aabakken'), '$125')
        self.assertEqual(self.pa.encode('Åbacken'), '$125')
        self.assertEqual(self.pa.encode('Ahlen'), '$45')
        self.assertEqual(self.pa.encode('Aleen'), '$45')
        self.assertEqual(self.pa.encode('Braunerhielm'), 'B656245')
        self.assertEqual(self.pa.encode('Branneerhielm'), 'B656245')
        self.assertEqual(self.pa.encode('Carlzon'), 'K6485')
        self.assertEqual(self.pa.encode('Karlsson'), 'K6485')
        self.assertEqual(self.pa.encode('Enochsson'), '$5285')
        self.assertEqual(self.pa.encode('Ericsson'), '$6285')
        self.assertEqual(self.pa.encode('Ericksson'), '$6285')
        self.assertEqual(self.pa.encode('Erixson'), '$6285')
        self.assertEqual(self.pa.encode('Filipsson'), 'F4185')
        self.assertEqual(self.pa.encode('Philipson'), 'F4185')
        self.assertEqual(self.pa.encode('Flycht'), 'F423')
        self.assertEqual(self.pa.encode('Flygt'), 'F423')
        self.assertEqual(self.pa.encode('Flykt'), 'F423')
        self.assertEqual(self.pa.encode('Fröijer'), 'F626')
        self.assertEqual(self.pa.encode('Fröjer'), 'F626')
        self.assertEqual(self.pa.encode('Gertner'), 'J6356')
        self.assertEqual(self.pa.encode('Hiertner'), 'J6356')
        self.assertEqual(self.pa.encode('Hirch'), 'H62')
        self.assertEqual(self.pa.encode('Hirsch'), 'H68')
        self.assertEqual(self.pa.encode('Haegermarck'), 'H26562')
        self.assertEqual(self.pa.encode('Hägermark'), 'H26562')
        self.assertEqual(self.pa.encode('Isaxon'), '$8285')
        self.assertEqual(self.pa.encode('Isacsson'), '$8285')
        self.assertEqual(self.pa.encode('Joachimsson'), 'J2585')
        self.assertEqual(self.pa.encode('Joakimson'), 'J2585')
        self.assertEqual(self.pa.encode('Kjell'), '#4')
        self.assertEqual(self.pa.encode('Käll'), '#4')
        self.assertEqual(self.pa.encode('Knapp'), 'K51')
        self.assertEqual(self.pa.encode('Krans'), 'K658')
        self.assertEqual(self.pa.encode('Krantz'), 'K6538')
        self.assertEqual(self.pa.encode('Kvist'), 'K783')
        self.assertEqual(self.pa.encode('Quist'), 'K783')
        self.assertEqual(self.pa.encode('Lidbeck'), 'L312')
        self.assertEqual(self.pa.encode('Lidbäck'), 'L312')
        self.assertEqual(self.pa.encode('Linnér'), 'L56')
        self.assertEqual(self.pa.encode('Linner'), 'L56')
        self.assertEqual(self.pa.encode('Lorenzsonn'), 'L6585')
        self.assertEqual(self.pa.encode('Lorentzon'), 'L65385')
        self.assertEqual(self.pa.encode('Lorenßon'), 'L6585')
        self.assertEqual(self.pa.encode('Lyxell'), 'L284')
        self.assertEqual(self.pa.encode('Lycksell'), 'L284')
        self.assertEqual(self.pa.encode('Marcström'), 'M628365')
        self.assertEqual(self.pa.encode('Markström'), 'M628365')
        self.assertEqual(self.pa.encode('Michaelsson'), 'M2485')
        self.assertEqual(self.pa.encode('Mikaelson'), 'M2485')
        self.assertEqual(self.pa.encode('Mörch'), 'M62')
        self.assertEqual(self.pa.encode('Mörck'), 'M62')
        self.assertEqual(self.pa.encode('Mörk'), 'M62')
        self.assertEqual(self.pa.encode('Mørk'), 'M62')
        self.assertEqual(self.pa.encode('Nääs'), 'N8')
        self.assertEqual(self.pa.encode('Naess'), 'N8')
        self.assertEqual(self.pa.encode('Nordstedt'), 'N63833')
        self.assertEqual(self.pa.encode('Oxenstierna'), '$28583265')
        self.assertEqual(self.pa.encode('Palmçrañtz'), 'P4526538')
        self.assertEqual(self.pa.encode('Palmcrantz'), 'P4526538')
        self.assertEqual(self.pa.encode('Palmkrantz'), 'P4526538')
        self.assertEqual(self.pa.encode('Preuss'), 'P68')
        self.assertEqual(self.pa.encode('Preutz'), 'P638')
        self.assertEqual(self.pa.encode('Richardson'), 'R26385')
        self.assertEqual(self.pa.encode('Rikardson'), 'R26385')
        self.assertEqual(self.pa.encode('Ruuth'), 'R3')
        self.assertEqual(self.pa.encode('Ruth'), 'R3')
        self.assertEqual(self.pa.encode('Sæter'), 'S36')
        self.assertEqual(self.pa.encode('Zäter'), 'S36')
        self.assertEqual(self.pa.encode('Schedin'), '#35')
        self.assertEqual(self.pa.encode('Sjödin'), '#35')
        self.assertEqual(self.pa.encode('Siöö'), '#')
        self.assertEqual(self.pa.encode('Sjöh'), '#')
        self.assertEqual(self.pa.encode('Svedberg'), 'S73162')
        self.assertEqual(self.pa.encode('Zwedberg'), 'S73162')
        self.assertEqual(self.pa.encode('Tjäder'), '#36')
        self.assertEqual(self.pa.encode('þornquist'), 'T652783')
        self.assertEqual(self.pa.encode('Thörnqvist'), 'T652783')
        self.assertEqual(self.pa.encode('Törnkvist'), 'T652783')
        self.assertEqual(self.pa.encode('Wichman'), 'V255')
        self.assertEqual(self.pa.encode('Wickman'), 'V255')
        self.assertEqual(self.pa.encode('Wictorin'), 'V2365')
        self.assertEqual(self.pa.encode('Wictorsson'), 'V23685')
        self.assertEqual(self.pa.encode('Viktorson'), 'V23685')
        self.assertEqual(self.pa.encode('Zachrisson'), 'S2685')
        self.assertEqual(self.pa.encode('Zakrison'), 'S2685')
        self.assertEqual(self.pa.encode('Övragård'), '$76263')
        self.assertEqual(self.pa.encode('Öfvragårdh'), '$76263')
        self.assertEqual(self.pa.encode('Bogdanovic'), 'B23572')
        self.assertEqual(self.pa.encode('Bogdanovitch'), 'B235732')
        self.assertEqual(self.pa.encode('Dieterich'), 'D362')
        self.assertEqual(self.pa.encode('Eichorn'), '$265')
        self.assertEqual(self.pa.encode('Friedrich'), 'F6362')
        self.assertEqual(self.pa.encode('Grantcharova'), 'G653267')
        self.assertEqual(self.pa.encode('Ilichev'), '$427')
        self.assertEqual(self.pa.encode('Ivankovic'), '$75272')
        self.assertEqual(self.pa.encode('Ivangurich'), '$75262')
        self.assertEqual(self.pa.encode('Kinch'), '#52')
        self.assertEqual(self.pa.encode('Kirchmann'), '#6255')
        self.assertEqual(self.pa.encode('Machado'), 'M23')
        self.assertEqual(self.pa.encode('Reich'), 'R2')
        self.assertEqual(self.pa.encode('Roche'), 'R2')
        self.assertEqual(self.pa.encode('Rubaszkin'), 'R1825')
        self.assertEqual(self.pa.encode('Rubaschkin'), 'R1825')
        self.assertEqual(self.pa.encode('Sanchez'), 'S528')
        self.assertEqual(self.pa.encode('Walukiewicz'), 'V42728')
        self.assertEqual(self.pa.encode('Valukievitch'), 'V42732')
        self.assertEqual(self.pa.encode('K'), 'K')
        self.assertEqual(self.pa.encode('2010'), '')
        self.assertEqual(self.pa.encode('cese'), 'S8')

        # a few max_length tests
        self.assertEqual(SfinxBis(3).encode('Kiviniemi Birgersson'), '#75,B62')
        self.assertEqual(self.pa4.encode('Eichorn'), '$265')
        self.assertEqual(self.pa4.encode('Friedrich'), 'F636')
        self.assertEqual(self.pa4.encode('Grantcharova'), 'G653')
        self.assertEqual(self.pa4.encode('Ilichev'), '$427')
        self.assertEqual(self.pa4.encode('Ivankovic'), '$752')
        self.assertEqual(self.pa4.encode('Ivangurich'), '$752')
        self.assertEqual(self.pa4.encode('Kinch'), '#52')
        self.assertEqual(self.pa4.encode('Kirchmann'), '#625')
        self.assertEqual(self.pa4.encode('Machado'), 'M23')
        self.assertEqual(self.pa4.encode('Reich'), 'R2')
        self.assertEqual(self.pa4.encode('Roche'), 'R2')
        self.assertEqual(self.pa4.encode('Rubaszkin'), 'R182')
        self.assertEqual(self.pa4.encode('Rubaschkin'), 'R182')
        self.assertEqual(self.pa4.encode('Sanchez'), 'S528')
        self.assertEqual(self.pa4.encode('Walukiewicz'), 'V427')
        self.assertEqual(self.pa4.encode('Valukievitch'), 'V427')
        self.assertEqual(self.pa4.encode('K'), 'K')
        self.assertEqual(self.pa4.encode('2010'), '')
        self.assertEqual(self.pa4.encode('cese'), 'S8')

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('chans'), '#58')
        self.assertEqual(self.pa.encode('ljud'), 'J3')
        self.assertEqual(self.pa.encode('qi'), 'K')
        self.assertEqual(self.pa.encode('xavier'), 'S76')
        self.assertEqual(self.pa.encode('skjul'), '#4')
        self.assertEqual(self.pa.encode('schul'), '#4')
        self.assertEqual(self.pa.encode('skil'), '#4')

        # max_length bounds tests
        self.assertEqual(SfinxBis(max_length=-1).encode('Niall'), 'N4')
        self.assertEqual(SfinxBis(max_length=0).encode('Niall'), 'N4')

        # encode_alpha
        self.assertEqual(
            self.pa.encode_alpha('Stael von Holstein'), 'STL,HLSTKN'
        )
        self.assertEqual(
            self.pa.encode_alpha('de Oliveira e Silva'), '$LFKR,SLF'
        )
        self.assertEqual(self.pa.encode_alpha('de Alfaro y Gómez'), '$LFR,GNS')
        self.assertEqual(
            self.pa.encode_alpha('Arjaliès-de la Lande'), '$RKLS,LNT'
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_phonem.

This module contains unit tests for abydos.phonetic.Phonem
"""

import unittest

from abydos.phonetic import Phonem


class PhonemTestCases(unittest.TestCase):
    """Test Phonem functions.

    test cases for abydos.phonetic.Phonem
    """

    pa = Phonem()

    def test_phonem(self):
        """Test abydos.phonetic.Phonem."""
        self.assertEqual(self.pa.encode(''), '')

        # http://phonetik.phil-fak.uni-koeln.de/fileadmin/home/ritters/Allgemeine_Dateien/Martin_Wilz.pdf
        self.assertEqual(self.pa.encode('müller'), 'MYLR')
        self.assertEqual(self.pa.encode('schmidt'), 'CMYD')
        self.assertEqual(self.pa.encode('schneider'), 'CNAYDR')
        self.assertEqual(self.pa.encode('fischer'), 'VYCR')
        self.assertEqual(self.pa.encode('weber'), 'VBR')
        self.assertEqual(self.pa.encode('meyer'), 'MAYR')
        self.assertEqual(self.pa.encode('wagner'), 'VACNR')
        self.assertEqual(self.pa.encode('schulz'), 'CULC')
        self.assertEqual(self.pa.encode('becker'), 'BCR')
        self.assertEqual(self.pa.encode('hoffmann'), 'OVMAN')
        self.assertEqual(self.pa.encode('schäfer'), 'CVR')

        # http://cpansearch.perl.org/src/MAROS/Text-Phonetic-2.05/t/008_phonem.t
        self.assertEqual(self.pa.encode('mair'), 'MAYR')
        self.assertEqual(self.pa.encode('bäker'), 'BCR')
        self.assertEqual(self.pa.encode('schaeffer'), 'CVR')
        self.assertEqual(self.pa.encode('computer'), 'COMBUDR')
        self.assertEqual(self.pa.encode('pfeifer'), 'VAYVR')
        self.assertEqual(self.pa.encode('pfeiffer'), 'VAYVR')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_extract.

This module contains unit tests for abydos.fingerprint.Extract
"""

import unittest

from abydos.fingerprint import Extract


class ExtractTestCases(unittest.TestCase):
    """Test Extract functions.

    abydos.fingerprint.Extract
    """

    fp = Extract()

    def test_extract_fingerprint(self):
        """Test abydos.fingerprint.Extract."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '')

        # Test cases from paper
        self.assertEqual(self.fp.fingerprint('Johnson'), 'JHNS')

        self.assertEqual(Extract(letter_list=2).fingerprint('Johnson'), 'JHNN')
        self.assertEqual(
            Extract(letter_list='ETASIONRHCDLPMFBUWGYKVJQZX').fingerprint(
                'Johnson'
            ),
            'JHNN',
        )
        self.assertEqual(Extract(letter_list=0).fingerprint('Johnson'), 'JHNS')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_tversky.

This module contains unit tests for abydos.distance.Tversky
"""

import unittest

from abydos.distance import Tversky
from abydos.tokenizer import QGrams, WhitespaceTokenizer

from .. import NONQ_FROM, NONQ_TO


class TverskyIndexTestCases(unittest.TestCase):
    """Test Tversky functions.

    abydos.distance.Tversky
    """

    cmp = Tversky()
    cmp_q2 = Tversky(tokenizer=QGrams(2))
    cmp_ws = Tversky(tokenizer=WhitespaceTokenizer())

    def test_tversky_sim(self):
        """Test abydos.distance.Tversky.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('nelson', ''), 0)
        self.assertEqual(self.cmp.sim('', 'neilsen'), 0)
        self.assertAlmostEqual(self.cmp.sim('nelson', 'neilsen'), 4 / 11)

        self.assertEqual(self.cmp_q2.sim('', ''), 1)
        self.assertEqual(self.cmp_q2.sim('nelson', ''), 0)
        self.assertEqual(self.cmp_q2.sim('', 'neilsen'), 0)
        self.assertAlmostEqual(self.cmp_q2.sim('nelson', 'neilsen'), 4 / 11)

        # test valid alpha & beta
        self.assertRaises(
            ValueError, Tversky(alpha=-1.0, beta=-1.0).sim, 'abcd', 'dcba'
        )
        self.assertRaises(
            ValueError, Tversky(alpha=-1.0, beta=0.0).sim, 'abcd', 'dcba'
        )
        self.assertRaises(
            ValueError, Tversky(alpha=0.0, beta=-1.0).sim, 'abcd', 'dcba'
        )

        # test empty QGrams
        self.assertAlmostEqual(
            Tversky(tokenizer=QGrams(7, start_stop='')).sim(
                'nelson', 'neilsen'
            ),
            0.0,
        )

        # test unequal alpha & beta
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=1.0, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            3 / 11,
        )
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=2.0, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            3 / 10,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=2.0, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            3 / 13,
        )

        # test bias parameter
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=1.0, bias=0.5, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            7 / 11,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=1.0, bias=0.5, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            7 / 9,
        )
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=2.0, bias=0.5, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            7 / 15,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=2.0, bias=0.5, tokenizer=QGrams(2)).sim(
                'niall', 'neal'
            ),
            7 / 11,
        )

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            0,
        )
        self.assertAlmostEqual(
            self.cmp.sim(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            4 / 11,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.sim('', ''), 1)
        self.assertEqual(self.cmp_ws.sim('the quick', ''), 0)
        self.assertEqual(self.cmp_ws.sim('', 'the quick'), 0)
        self.assertAlmostEqual(self.cmp_ws.sim(NONQ_FROM, NONQ_TO), 1 / 3)
        self.assertAlmostEqual(self.cmp_ws.sim(NONQ_TO, NONQ_FROM), 1 / 3)

    def test_tversky_dist(self):
        """Test abydos.distance.Tversky.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('nelson', ''), 1)
        self.assertEqual(self.cmp.dist('', 'neilsen'), 1)
        self.assertAlmostEqual(self.cmp.dist('nelson', 'neilsen'), 7 / 11)

        self.assertEqual(self.cmp_q2.dist('', ''), 0)
        self.assertEqual(self.cmp_q2.dist('nelson', ''), 1)
        self.assertEqual(self.cmp_q2.dist('', 'neilsen'), 1)
        self.assertAlmostEqual(self.cmp_q2.dist('nelson', 'neilsen'), 7 / 11)

        # test valid alpha & beta
        self.assertRaises(
            ValueError, Tversky(alpha=-1.0, beta=-1.0).dist, 'abcd', 'dcba'
        )
        self.assertRaises(
            ValueError, Tversky(alpha=-1.0, beta=0.0).dist, 'abcd', 'dcba'
        )
        self.assertRaises(
            ValueError, Tversky(alpha=0.0, beta=-1.0).dist, 'abcd', 'dcba'
        )

        # test empty QGrams
        self.assertAlmostEqual(
            Tversky(tokenizer=QGrams(7, start_stop='')).dist(
                'nelson', 'neilsen'
            ),
            1.0,
        )

        # test unequal alpha & beta
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=1.0, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            8 / 11,
        )
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=2.0, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            7 / 10,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=2.0, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            10 / 13,
        )

        # test bias parameter
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=1.0, bias=0.5, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            4 / 11,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=1.0, bias=0.5, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            2 / 9,
        )
        self.assertAlmostEqual(
            Tversky(alpha=1.0, beta=2.0, bias=0.5, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            8 / 15,
        )
        self.assertAlmostEqual(
            Tversky(alpha=2.0, beta=2.0, bias=0.5, tokenizer=QGrams(2)).dist(
                'niall', 'neal'
            ),
            4 / 11,
        )

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            1,
        )
        self.assertAlmostEqual(
            self.cmp.dist(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            7 / 11,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.dist('', ''), 0)
        self.assertEqual(self.cmp_ws.dist('the quick', ''), 1)
        self.assertEqual(self.cmp_ws.dist('', 'the quick'), 1)
        self.assertAlmostEqual(self.cmp_ws.dist(NONQ_FROM, NONQ_TO), 2 / 3)
        self.assertAlmostEqual(self.cmp_ws.dist(NONQ_TO, NONQ_FROM), 2 / 3)


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.fingerprint._count.

Cisłak & Grabowski's count fingerprint
"""

from collections import Counter
from typing import Tuple

from ._fingerprint import MOST_COMMON_LETTERS_CG, _Fingerprint

__all__ = ['Count']


class Count(_Fingerprint):
    """Count Fingerprint.

    Based on the count fingerprint from :cite:`Cislak:2017`.

    .. versionadded:: 0.3.6
    """

    def __init__(
        self,
        n_bits: int = 16,
        most_common: Tuple[str, ...] = MOST_COMMON_LETTERS_CG,
    ) -> None:
        """Initialize Count instance.

        Parameters
        ----------
        n_bits : int
            Number of bits in the fingerprint returned
        most_common : list
            The most common tokens in the target language, ordered by frequency


        .. versionadded:: 0.4.0

        """
        super(Count, self).__init__()
        self._n_bits = n_bits
        self._most_common = most_common

    def fingerprint(self, word: str) -> str:
        """Return the count fingerprint.

        Parameters
        ----------
        word : str
            The word to fingerprint

        Returns
        -------
        str
            The count fingerprint

        Examples
        --------
        >>> cf = Count()
        >>> cf.fingerprint('hat')
        '0001010000000001'
        >>> cf.fingerprint('niall')
        '0000010001010000'
        >>> cf.fingerprint('colin')
        '0000000101010000'
        >>> cf.fingerprint('atcg')
        '0001010000000000'
        >>> cf.fingerprint('entreatment')
        '1111010000100000'


        .. versionadded:: 0.3.0
        .. versionchanged:: 0.3.6
            Encapsulated in class
        .. versionchanged:: 0.6.0
            Changed to return a str and added fingerprint_int method

        """
        return ('{:0' + str(self._n_bits) + 'b}').format(
            self.fingerprint_int(word)
        )

    def fingerprint_int(self, word: str) -> int:
        """Return the count fingerprint.

        Parameters
        ----------
        word : str
            The word to fingerprint

        Returns
        -------
        int
            The count fingerprint as an int

        Examples
        --------
        >>> cf = Count()
        >>> cf.fingerprint_int('hat')
        5121
        >>> cf.fingerprint_int('niall')
        1104
        >>> cf.fingerprint_int('colin')
        336
        >>> cf.fingerprint_int('atcg')
        5120
        >>> cf.fingerprint_int('entreatment')
        62496


        .. versionadded:: 0.6.0

        """
        n_bits = self._n_bits
        if n_bits % 2:
            n_bits += 1

        letter_counts = Counter(word)
        fingerprint = 0

        for letter in self._most_common:
            if n_bits:
                fingerprint <<= 2
                fingerprint += letter_counts[letter] & 3
                n_bits -= 2
            else:
                break

        if n_bits:
            fingerprint <<= n_bits

        return fingerprint


if __name__ == '__main__':
    import doctest

    doctest.testmod()



import builtins
import sys
import datetime as dt

from numpy.core._internal import _ctypes
from typing import (
    Any,
    ByteString,
    Container,
    Dict,
    IO,
    Iterable,
    List,
    Mapping,
    Optional,
    overload,
    Sequence,
    Sized,
    SupportsAbs,
    SupportsComplex,
    SupportsFloat,
    SupportsInt,
    Text,
    Tuple,
    Type,
    TypeVar,
    Union,
)

if sys.version_info[0] < 3:
    class SupportsBytes: ...

else:
    from typing import SupportsBytes

_Shape = Tuple[int, ...]

# Anything that can be coerced to a shape tuple
_ShapeLike = Union[int, Sequence[int]]

_DtypeLikeNested = Any  # TODO: wait for support for recursive types

# Anything that can be coerced into numpy.dtype.
# Reference: https://docs.scipy.org/doc/numpy/reference/arrays.dtypes.html
_DtypeLike = Union[
    dtype,
    # default data type (float64)
    None,
    # array-scalar types and generic types
    type,  # TODO: enumerate these when we add type hints for numpy scalars
    # TODO: add a protocol for anything with a dtype attribute
    # character codes, type strings or comma-separated fields, e.g., 'float64'
    str,
    # (flexible_dtype, itemsize)
    Tuple[_DtypeLikeNested, int],
    # (fixed_dtype, shape)
    Tuple[_DtypeLikeNested, _ShapeLike],
    # [(field_name, field_dtype, field_shape), ...]
    List[
        Union[
            Tuple[Union[str, Tuple[str, str]], _DtypeLikeNested],
            Tuple[Union[str, Tuple[str, str]], _DtypeLikeNested, _ShapeLike],
        ]
    ],
    # {'names': ..., 'formats': ..., 'offsets': ..., 'titles': ...,
    #  'itemsize': ...}
    # TODO: use TypedDict when/if it's officially supported
    Dict[
        str,
        Union[
            Sequence[str],  # names
            Sequence[_DtypeLikeNested],  # formats
            Sequence[int],  # offsets
            Sequence[Union[bytes, Text, None]],  # titles
            int,  # itemsize
        ],
    ],
    # {'field1': ..., 'field2': ..., ...}
    Dict[str, Tuple[_DtypeLikeNested, int]],
    # (base_dtype, new_dtype)
    Tuple[_DtypeLikeNested, _DtypeLikeNested],
]

_NdArraySubClass = TypeVar("_NdArraySubClass", bound=ndarray)

_ArrayLike = TypeVar("_ArrayLike")

class dtype:
    names: Optional[Tuple[str, ...]]
    def __init__(
        self, obj: _DtypeLike, align: bool = ..., copy: bool = ...
    ) -> None: ...
    @property
    def alignment(self) -> int: ...
    @property
    def base(self) -> dtype: ...
    @property
    def byteorder(self) -> str: ...
    @property
    def char(self) -> str: ...
    @property
    def descr(
        self,
    ) -> List[Union[Tuple[str, str], Tuple[str, str, _Shape]]]: ...
    @property
    def fields(
        self,
    ) -> Optional[
        Mapping[str, Union[Tuple[dtype, int], Tuple[dtype, int, Any]]]
    ]: ...
    @property
    def flags(self) -> int: ...
    @property
    def hasobject(self) -> bool: ...
    @property
    def isbuiltin(self) -> int: ...
    @property
    def isnative(self) -> bool: ...
    @property
    def isalignedstruct(self) -> bool: ...
    @property
    def itemsize(self) -> int: ...
    @property
    def kind(self) -> str: ...
    @property
    def metadata(self) -> Optional[Mapping[str, Any]]: ...
    @property
    def name(self) -> str: ...
    @property
    def num(self) -> int: ...
    @property
    def shape(self) -> _Shape: ...
    @property
    def ndim(self) -> int: ...
    @property
    def subdtype(self) -> Optional[Tuple[dtype, _Shape]]: ...
    def newbyteorder(self, new_order: str = ...) -> dtype: ...
    # Leave str and type for end to avoid having to use `builtins.str`
    # everywhere. See https://github.com/python/mypy/issues/3775
    @property
    def str(self) -> builtins.str: ...
    @property
    def type(self) -> Type[generic]: ...

_Dtype = dtype  # to avoid name conflicts with ndarray.dtype

class _flagsobj:
    aligned: bool
    updateifcopy: bool
    writeable: bool
    writebackifcopy: bool
    @property
    def behaved(self) -> bool: ...
    @property
    def c_contiguous(self) -> bool: ...
    @property
    def carray(self) -> bool: ...
    @property
    def contiguous(self) -> bool: ...
    @property
    def f_contiguous(self) -> bool: ...
    @property
    def farray(self) -> bool: ...
    @property
    def fnc(self) -> bool: ...
    @property
    def forc(self) -> bool: ...
    @property
    def fortran(self) -> bool: ...
    @property
    def num(self) -> int: ...
    @property
    def owndata(self) -> bool: ...
    def __getitem__(self, key: str) -> bool: ...
    def __setitem__(self, key: str, value: bool) -> None: ...

class flatiter:
    @property
    def base(self) -> ndarray: ...
    @property
    def coords(self) -> _Shape: ...
    @property
    def index(self) -> int: ...
    def copy(self) -> ndarray: ...
    def __iter__(self) -> flatiter: ...
    def __next__(self) -> Any: ...

_ArraySelf = TypeVar("_ArraySelf", bound=_ArrayOrScalarCommon)

class _ArrayOrScalarCommon(
    SupportsInt,
    SupportsFloat,
    SupportsComplex,
    SupportsBytes,
    SupportsAbs[Any],
):
    @property
    def T(self: _ArraySelf) -> _ArraySelf: ...
    @property
    def base(self) -> Optional[ndarray]: ...
    @property
    def dtype(self) -> _Dtype: ...
    @property
    def data(self) -> memoryview: ...
    @property
    def flags(self) -> _flagsobj: ...
    @property
    def size(self) -> int: ...
    @property
    def itemsize(self) -> int: ...
    @property
    def nbytes(self) -> int: ...
    @property
    def ndim(self) -> int: ...
    @property
    def shape(self) -> _Shape: ...
    @property
    def strides(self) -> _Shape: ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __complex__(self) -> complex: ...
    if sys.version_info[0] < 3:
        def __oct__(self) -> str: ...
        def __hex__(self) -> str: ...
        def __nonzero__(self) -> bool: ...
        def __unicode__(self) -> Text: ...
    else:
        def __bool__(self) -> bool: ...
        def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __copy__(self: _ArraySelf, order: str = ...) -> _ArraySelf: ...
    def __deepcopy__(self: _ArraySelf, memo: dict) -> _ArraySelf: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __add__(self, other): ...
    def __radd__(self, other): ...
    def __iadd__(self, other): ...
    def __sub__(self, other): ...
    def __rsub__(self, other): ...
    def __isub__(self, other): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __imul__(self, other): ...
    if sys.version_info[0] < 3:
        def __div__(self, other): ...
        def __rdiv__(self, other): ...
        def __idiv__(self, other): ...
    def __truediv__(self, other): ...
    def __rtruediv__(self, other): ...
    def __itruediv__(self, other): ...
    def __floordiv__(self, other): ...
    def __rfloordiv__(self, other): ...
    def __ifloordiv__(self, other): ...
    def __mod__(self, other): ...
    def __rmod__(self, other): ...
    def __imod__(self, other): ...
    def __divmod__(self, other): ...
    def __rdivmod__(self, other): ...
    # NumPy's __pow__ doesn't handle a third argument
    def __pow__(self, other): ...
    def __rpow__(self, other): ...
    def __ipow__(self, other): ...
    def __lshift__(self, other): ...
    def __rlshift__(self, other): ...
    def __ilshift__(self, other): ...
    def __rshift__(self, other): ...
    def __rrshift__(self, other): ...
    def __irshift__(self, other): ...
    def __and__(self, other): ...
    def __rand__(self, other): ...
    def __iand__(self, other): ...
    def __xor__(self, other): ...
    def __rxor__(self, other): ...
    def __ixor__(self, other): ...
    def __or__(self, other): ...
    def __ror__(self, other): ...
    def __ior__(self, other): ...
    if sys.version_info[:2] >= (3, 5):
        def __matmul__(self, other): ...
        def __rmatmul__(self, other): ...
    def __neg__(self: _ArraySelf) -> _ArraySelf: ...
    def __pos__(self: _ArraySelf) -> _ArraySelf: ...
    def __abs__(self: _ArraySelf) -> _ArraySelf: ...
    def __invert__(self: _ArraySelf) -> _ArraySelf: ...
    # TODO(shoyer): remove when all methods are defined
    def __getattr__(self, name) -> Any: ...

_BufferType = Union[ndarray, bytes, bytearray, memoryview]

class ndarray(_ArrayOrScalarCommon, Iterable, Sized, Container):
    real: ndarray
    imag: ndarray
    def __new__(
        cls,
        shape: Sequence[int],
        dtype: Union[_DtypeLike, str] = ...,
        buffer: _BufferType = ...,
        offset: int = ...,
        strides: _ShapeLike = ...,
        order: Optional[str] = ...,
    ) -> ndarray: ...
    @property
    def dtype(self) -> _Dtype: ...
    @dtype.setter
    def dtype(self, value: _DtypeLike): ...
    @property
    def ctypes(self) -> _ctypes: ...
    @property
    def shape(self) -> _Shape: ...
    @shape.setter
    def shape(self, value: _ShapeLike): ...
    @property
    def flat(self) -> flatiter: ...
    @property
    def strides(self) -> _Shape: ...
    @strides.setter
    def strides(self, value: _ShapeLike): ...
    # Array conversion
    @overload
    def item(self, *args: int) -> Any: ...
    @overload
    def item(self, args: Tuple[int, ...]) -> Any: ...
    def tolist(self) -> List[Any]: ...
    @overload
    def itemset(self, __value: Any) -> None: ...
    @overload
    def itemset(self, __item: _ShapeLike, __value: Any) -> None: ...
    def tostring(self, order: Optional[str] = ...) -> bytes: ...
    def tobytes(self, order: Optional[str] = ...) -> bytes: ...
    def tofile(
        self, fid: Union[IO[bytes], str], sep: str = ..., format: str = ...
    ) -> None: ...
    def dump(self, file: str) -> None: ...
    def dumps(self) -> bytes: ...
    def astype(
        self,
        dtype: _DtypeLike,
        order: str = ...,
        casting: str = ...,
        subok: bool = ...,
        copy: bool = ...,
    ) -> ndarray: ...
    def byteswap(self, inplace: bool = ...) -> ndarray: ...
    def copy(self, order: str = ...) -> ndarray: ...
    @overload
    def view(self, dtype: Type[_NdArraySubClass]) -> _NdArraySubClass: ...
    @overload
    def view(self, dtype: _DtypeLike = ...) -> ndarray: ...
    @overload
    def view(
        self, dtype: _DtypeLike, type: Type[_NdArraySubClass]
    ) -> _NdArraySubClass: ...
    @overload
    def view(self, *, type: Type[_NdArraySubClass]) -> _NdArraySubClass: ...
    def getfield(
        self, dtype: Union[_DtypeLike, str], offset: int = ...
    ) -> ndarray: ...
    def setflags(
        self, write: bool = ..., align: bool = ..., uic: bool = ...
    ) -> None: ...
    def fill(self, value: Any) -> None: ...
    # Shape manipulation
    @overload
    def reshape(
        self, shape: Sequence[int], *, order: str = ...
    ) -> ndarray: ...
    @overload
    def reshape(self, *shape: int, order: str = ...) -> ndarray: ...
    @overload
    def resize(
        self, new_shape: Sequence[int], *, refcheck: bool = ...
    ) -> None: ...
    @overload
    def resize(self, *new_shape: int, refcheck: bool = ...) -> None: ...
    @overload
    def transpose(self, axes: Sequence[int]) -> ndarray: ...
    @overload
    def transpose(self, *axes: int) -> ndarray: ...
    def swapaxes(self, axis1: int, axis2: int) -> ndarray: ...
    def flatten(self, order: str = ...) -> ndarray: ...
    def ravel(self, order: str = ...) -> ndarray: ...
    def squeeze(self, axis: Union[int, Tuple[int, ...]] = ...) -> ndarray: ...
    # Many of these special methods are irrelevant currently, since protocols
    # aren't supported yet. That said, I'm adding them for completeness.
    # https://docs.python.org/3/reference/datamodel.html
    def __len__(self) -> int: ...
    def __getitem__(self, key) -> Any: ...
    def __setitem__(self, key, value): ...
    def __iter__(self) -> Any: ...
    def __contains__(self, key) -> bool: ...
    def __index__(self) -> int: ...

class generic(_ArrayOrScalarCommon):
    def __init__(self, value: Any = ...) -> None: ...
    @property
    def base(self) -> None: ...

class _real_generic(generic):
    @property
    def real(self: _ArraySelf) -> _ArraySelf: ...
    @property
    def imag(self: _ArraySelf) -> _ArraySelf: ...

class number(generic):
    def __init__(
        self, value: Union[SupportsInt, SupportsFloat] = ...
    ) -> None: ...

class bool_(_real_generic): ...
class object_(generic): ...

class datetime64:
    @overload
    def __init__(
        self,
        _data: Union[datetime64, str, dt.datetime] = ...,
        _format: str = ...,
    ) -> None: ...
    @overload
    def __init__(self, _data: int, _format: str) -> None: ...
    def __add__(self, other: Union[timedelta64, int]) -> datetime64: ...
    def __sub__(
        self, other: Union[timedelta64, datetime64, int]
    ) -> timedelta64: ...

class integer(number, _real_generic): ...
class signedinteger(integer): ...
class int8(signedinteger): ...
class int16(signedinteger): ...
class int32(signedinteger): ...
class int64(signedinteger): ...

class timedelta64(signedinteger):
    def __init__(self, _data: Any = ..., _format: str = ...) -> None: ...
    @overload
    def __add__(self, other: Union[timedelta64, int]) -> timedelta64: ...
    @overload
    def __add__(self, other: datetime64) -> datetime64: ...
    def __sub__(self, other: Union[timedelta64, int]) -> timedelta64: ...
    if sys.version_info[0] < 3:
        @overload
        def __div__(self, other: timedelta64) -> float: ...
        @overload
        def __div__(self, other: float) -> timedelta64: ...
    @overload
    def __truediv__(self, other: timedelta64) -> float: ...
    @overload
    def __truediv__(self, other: float) -> timedelta64: ...
    def __mod__(self, other: timedelta64) -> timedelta64: ...

class unsignedinteger(integer): ...
class uint8(unsignedinteger): ...
class uint16(unsignedinteger): ...
class uint32(unsignedinteger): ...
class uint64(unsignedinteger): ...
class inexact(number): ...
class floating(inexact, _real_generic): ...
class float16(floating): ...
class float32(floating): ...
class float64(floating): ...

class complexfloating(inexact):
    def __init__(
        self,
        value: Union[
            SupportsInt, SupportsFloat, SupportsComplex, complex
        ] = ...,
    ) -> None: ...

class complex64(complexfloating):
    @property
    def real(self) -> float32: ...
    @property
    def imag(self) -> float32: ...

class complex128(complexfloating):
    @property
    def real(self) -> float64: ...
    @property
    def imag(self) -> float64: ...

class flexible(_real_generic): ...
class void(flexible): ...
class character(_real_generic): ...
class bytes_(character): ...
class str_(character): ...

# TODO(alan): Platform dependent types
# longcomplex, longdouble, longfloat
# bytes, short, intc, intp, longlong
# half, single, double, longdouble
# uint_, int_, float_, complex_
# float128, complex256
# float96

def array(
    object: object,
    dtype: _DtypeLike = ...,
    copy: bool = ...,
    subok: bool = ...,
    ndmin: int = ...,
) -> ndarray: ...
def zeros(
    shape: _ShapeLike, dtype: _DtypeLike = ..., order: Optional[str] = ...
) -> ndarray: ...
def ones(
    shape: _ShapeLike, dtype: _DtypeLike = ..., order: Optional[str] = ...
) -> ndarray: ...

# TODO(shoyer): remove when the full numpy namespace is defined
def __getattr__(name: str) -> Any: ...



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_caverphone.

This module contains unit tests for abydos.phonetic.Caverphone
"""

import unittest

from abydos.phonetic import Caverphone, Metaphone, Soundex

from .. import _corpus_file


class CaverphoneTestCases(unittest.TestCase):
    """Test Caverphone functions.

    test cases for abydos.phonetic.Caverphone
    """

    pa = Caverphone()
    pa_1 = Caverphone(version=1)
    pa_2 = Caverphone(version=2)

    def test_caverphone2_encode(self):
        """Test abydos.phonetic.Caverphone (Caverphone 2)."""
        self.assertEqual(self.pa.encode(''), '1111111111')
        self.assertEqual(self.pa_2.encode(''), '1111111111')

        # http://ntz-develop.blogspot.com/2011/03/phonetic-algorithms.html
        self.assertEqual(self.pa.encode('Henrichsen'), 'ANRKSN1111')
        self.assertEqual(self.pa.encode('Henricsson'), 'ANRKSN1111')
        self.assertEqual(self.pa.encode('Henriksson'), 'ANRKSN1111')
        self.assertEqual(self.pa.encode('Hinrichsen'), 'ANRKSN1111')
        self.assertEqual(self.pa.encode('Izchaki'), 'ASKKA11111')
        self.assertEqual(self.pa.encode('Maclaverty'), 'MKLFTA1111')
        self.assertEqual(self.pa.encode('Mccleverty'), 'MKLFTA1111')
        self.assertEqual(self.pa.encode('Mcclifferty'), 'MKLFTA1111')
        self.assertEqual(self.pa.encode('Mclafferty'), 'MKLFTA1111')
        self.assertEqual(self.pa.encode('Mclaverty'), 'MKLFTA1111')
        self.assertEqual(self.pa.encode('Slocomb'), 'SLKM111111')
        self.assertEqual(self.pa.encode('Slocombe'), 'SLKM111111')
        self.assertEqual(self.pa.encode('Slocumb'), 'SLKM111111')
        self.assertEqual(self.pa.encode('Whitlam'), 'WTLM111111')

        # http://caversham.otago.ac.nz/files/working/ctp150804.pdf
        self.assertEqual(self.pa.encode('Stevenson'), 'STFNSN1111')
        self.assertEqual(self.pa.encode('Peter'), 'PTA1111111')
        for word in (
            'Darda',
            'Datha',
            'Dedie',
            'Deedee',
            'Deerdre',
            'Deidre',
            'Deirdre',
            'Detta',
            'Didi',
            'Didier',
            'Dido',
            'Dierdre',
            'Dieter',
            'Dita',
            'Ditter',
            'Dodi',
            'Dodie',
            'Dody',
            'Doherty',
            'Dorthea',
            'Dorthy',
            'Doti',
            'Dotti',
            'Dottie',
            'Dotty',
            'Doty',
            'Doughty',
            'Douty',
            'Dowdell',
            'Duthie',
            'Tada',
            'Taddeo',
            'Tadeo',
            'Tadio',
            'Tati',
            'Teador',
            'Tedda',
            'Tedder',
            'Teddi',
            'Teddie',
            'Teddy',
            'Tedi',
            'Tedie',
            'Teeter',
            'Teodoor',
            'Teodor',
            'Terti',
            'Theda',
            'Theodor',
            'Theodore',
            'Theta',
            'Thilda',
            'Thordia',
            'Tilda',
            'Tildi',
            'Tildie',
            'Tildy',
            'Tita',
            'Tito',
            'Tjader',
            'Toddie',
            'Toddy',
            'Torto',
            'Tuddor',
            'Tudor',
            'Turtle',
            'Tuttle',
            'Tutto',
        ):
            self.assertEqual(self.pa.encode(word), 'TTA1111111')
            self.assertEqual(self.pa_2.encode(word), 'TTA1111111')
        for word in (
            'Cailean',
            'Calan',
            'Calen',
            'Callahan',
            'Callan',
            'Callean',
            'Carleen',
            'Carlen',
            'Carlene',
            'Carlin',
            'Carline',
            'Carlyn',
            'Carlynn',
            'Carlynne',
            'Charlean',
            'Charleen',
            'Charlene',
            'Charline',
            'Cherlyn',
            'Chirlin',
            'Clein',
            'Cleon',
            'Cline',
            'Cohleen',
            'Colan',
            'Coleen',
            'Colene',
            'Colin',
            'Colleen',
            'Collen',
            'Collin',
            'Colline',
            'Colon',
            'Cullan',
            'Cullen',
            'Cullin',
            'Gaelan',
            'Galan',
            'Galen',
            'Garlan',
            'Garlen',
            'Gaulin',
            'Gayleen',
            'Gaylene',
            'Giliane',
            'Gillan',
            'Gillian',
            'Glen',
            'Glenn',
            'Glyn',
            'Glynn',
            'Gollin',
            'Gorlin',
            'Kalin',
            'Karlan',
            'Karleen',
            'Karlen',
            'Karlene',
            'Karlin',
            'Karlyn',
            'Kaylyn',
            'Keelin',
            'Kellen',
            'Kellene',
            'Kellyann',
            'Kellyn',
            'Khalin',
            'Kilan',
            'Kilian',
            'Killen',
            'Killian',
            'Killion',
            'Klein',
            'Kleon',
            'Kline',
            'Koerlin',
            'Kylen',
            'Kylynn',
            'Quillan',
            'Quillon',
            'Qulllon',
            'Xylon',
        ):
            self.assertEqual(self.pa.encode(word), 'KLN1111111')
            self.assertEqual(self.pa_2.encode(word), 'KLN1111111')
        for word in (
            'Dan',
            'Dane',
            'Dann',
            'Darn',
            'Daune',
            'Dawn',
            'Ddene',
            'Dean',
            'Deane',
            'Deanne',
            'DeeAnn',
            'Deeann',
            'Deeanne',
            'Deeyn',
            'Den',
            'Dene',
            'Denn',
            'Deonne',
            'Diahann',
            'Dian',
            'Diane',
            'Diann',
            'Dianne',
            'Diannne',
            'Dine',
            'Dion',
            'Dione',
            'Dionne',
            'Doane',
            'Doehne',
            'Don',
            'Donn',
            'Doone',
            'Dorn',
            'Down',
            'Downe',
            'Duane',
            'Dun',
            'Dunn',
            'Duyne',
            'Dyan',
            'Dyane',
            'Dyann',
            'Dyanne',
            'Dyun',
            'Tan',
            'Tann',
            'Teahan',
            'Ten',
            'Tenn',
            'Terhune',
            'Thain',
            'Thaine',
            'Thane',
            'Thanh',
            'Thayne',
            'Theone',
            'Thin',
            'Thorn',
            'Thorne',
            'Thun',
            'Thynne',
            'Tien',
            'Tine',
            'Tjon',
            'Town',
            'Towne',
            'Turne',
            'Tyne',
        ):
            self.assertEqual(self.pa.encode(word), 'TN11111111')
            self.assertEqual(self.pa_2.encode(word), 'TN11111111')

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('enough'), 'ANF1111111')
        self.assertEqual(self.pa.encode('trough'), 'TRF1111111')
        self.assertEqual(self.pa.encode('gnu'), 'NA11111111')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Henrichsen'), 'ANRKSN')
        self.assertEqual(self.pa.encode_alpha('Dierdre'), 'TTA')
        self.assertEqual(self.pa.encode_alpha('Mcclifferty'), 'MKLFTA')
        self.assertEqual(self.pa.encode_alpha('Killen'), 'KLN')
        self.assertEqual(self.pa.encode_alpha('Whittle'), 'WTA')

    def test_caverphone2_encode_php_testset(self):
        """Test abydos.phonetic.Caverphone (PHP version testset)."""
        # https://raw.githubusercontent.com/kiphughes/caverphone/master/unit_tests.php
        with open(_corpus_file('php_caverphone.csv')) as php_testset:
            for php_line in php_testset:
                (word, caver) = php_line.strip().split(',')
                self.assertEqual(self.pa.encode(word), caver)

    def test_caverphone1_encode(self):
        """Test abydos.phonetic.Caverphone (Caverphone 1)."""
        self.assertEqual(self.pa_1.encode(''), '111111')

        # http://caversham.otago.ac.nz/files/working/ctp060902.pdf
        self.assertEqual(self.pa_1.encode('David'), 'TFT111')
        self.assertEqual(self.pa_1.encode('Whittle'), 'WTL111')

    def test_caversham(self):
        """Test using Caversham test set (SoundEx, Metaphone, & Caverphone)."""
        soundex = Soundex()
        metaphone = Metaphone()

        with open(_corpus_file('variantNames.csv')) as cav_testset:
            next(cav_testset)
            for cav_line in cav_testset:
                (
                    name1,
                    soundex1,
                    metaphone1,
                    caverphone1,
                    name2,
                    soundex2,
                    metaphone2,
                    caverphone2,
                    soundex_same,
                    metaphone_same,
                    caverphone_same,
                ) = cav_line.strip().split(',')

                self.assertEqual(soundex.encode(name1), soundex1)
                self.assertEqual(soundex.encode(name2), soundex2)
                if soundex_same == '1':
                    self.assertEqual(
                        soundex.encode(name1), soundex.encode(name2)
                    )
                else:
                    self.assertNotEqual(
                        soundex.encode(name1), soundex.encode(name2)
                    )

                self.assertEqual(metaphone.encode(name1), metaphone1)
                self.assertEqual(metaphone.encode(name2), metaphone2)
                if metaphone_same == '1':
                    self.assertEqual(
                        metaphone.encode(name1), metaphone.encode(name2)
                    )
                else:
                    self.assertNotEqual(
                        metaphone.encode(name1), metaphone.encode(name2)
                    )

                self.assertEqual(self.pa.encode(name1), caverphone1)
                self.assertEqual(self.pa.encode(name2), caverphone2)
                if caverphone_same == '1':
                    self.assertEqual(
                        self.pa.encode(name1), self.pa.encode(name2)
                    )
                else:
                    self.assertNotEqual(
                        self.pa.encode(name1), self.pa.encode(name2)
                    )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.


"""bm_php2py.py.

This helper script converts Beider-Morse Phonetic Matching Algorithm (BMPM)
code from PHP to Python.

It assumes that the BMPM code is located at ../../bmpm (relative to this
directory in the abydos repository).

It reads the BMPM reference implementation and generates the file
../abydos/_beider_morse_data.py.

The file _beider_morse.py may still need manual changes to be made after this
script is run.
"""

import codecs
import re
import sys
from os import listdir
from os.path import isfile

# noinspection PyPackageRequirements
import chardet

# The list of languages from BMPM to support (might need to be updated or
# tuned as BMPM is updated)
lang_tuple = (
    'any',
    'arabic',
    'cyrillic',
    'czech',
    'dutch',
    'english',
    'french',
    'german',
    'greek',
    'greeklatin',
    'hebrew',
    'hungarian',
    'italian',
    'latvian',
    'polish',
    'portuguese',
    'romanian',
    'russian',
    'spanish',
    'turkish',
)

lang_dict = {}
for i, l in enumerate(lang_tuple):
    lang_dict[l] = 2 ** i
lang_dict['common'] = "'common'"

nl = False
array_seen = False

tail_text = ''
sd = ''


def c2u(name):
    """Convert camelCase (used in PHP) to Python-standard snake_case.

    Src:
    https://stackoverflow.com/questions/1175208/elegant-python-function-to-convert-camelcase-to-snake-case

    Parameters
    ----------
    name: A function or variable name in camelCase

    Returns
    -------
    str: The name in snake_case

    """
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    s1 = re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
    return s1


def pythonize(line, fn='', subdir='gen'):
    """Convert a line of BMPM code from PHP to Python.

    Parameters
    ----------
    line : str
        A line of code
        fn : str
        A filename
        subdir : str
        The file's subdirectory

    Returns
    -------
    The code in Python

    """
    global array_seen, nl, sd

    if '$all' in line:
        return ''
    if 'make the sum of all languages be visible in the function' in line:
        return ''

    line = line.strip()

    if 'array' in line and not line.startswith('//'):
        array_seen = True

    line = re.sub('//+', '#', line)
    # line = re.sub('"\.\((\$.+?)\)\."', r'\1', line)
    if line and re.search(r'array\("[^"]+?"\)', line):
        # print("### " + line)
        line = ''
    line = line.replace('array', '')
    line = re.sub(r'^\s*', '', line)
    line = re.sub(';$', '', line)
    line = re.sub('^include_.+', '', line)

    line = re.sub(
        r'\$(approx|rules|exact)\[LanguageIndex\("([^"]+)", '
        + r'\$languages\)\] = \$([a-zA-Z]+)',
        lambda m: (
            "BMDATA['"
            + subdir
            + "']['"
            + m.group(1)
            + "'][L_"
            + m.group(2).upper()
            + '] = _'
            + subdir.upper()
            + '_'
            + c2u(m.group(3)).upper()
        ),
        line,
    )

    line = re.sub(
        r'\$(approx|rules|exact|hebrew)([A-Za-z]+) = _merge'
        + r'\(\$([a-zA-Z]+), \$([a-zA-Z]+)\)',
        lambda m: (
            "BMDATA['"
            + subdir
            + "']['"
            + m.group(1)
            + "'][L_"
            + c2u(m.group(2)).upper()
            + '] = _'
            + subdir.upper()
            + '_'
            + c2u(m.group(3)).upper()
            + ' + _'
            + subdir.upper()
            + '_'
            + c2u(m.group(4)).upper()
        ),
        line,
    )

    line = re.sub(
        r'\$(approx|rules|exact)\[LanguageIndex\("([^"]+)", '
        + r'\$languages\)\] = _merge\(\$([a-zA-Z]+), \$([a-zA-Z]+)\)',
        lambda m: (
            "BMDATA['"
            + subdir
            + "']['"
            + m.group(1)
            + "'][L_"
            + c2u(m.group(2)).upper()
            + '] = _'
            + subdir.upper()
            + '_'
            + c2u(m.group(3)).upper()
            + ' + _'
            + subdir.upper()
            + '_'
            + c2u(m.group(4)).upper()
        ),
        line,
    )

    line = re.sub(
        r'^\$([a-zA-Z]+)',
        lambda m: '_' + sd.upper() + '_' + c2u(m.group(1)).upper(),
        line,
    )

    for _ in range(len(lang_tuple)):
        line = re.sub(r'($[a-zA-Z]+) *\+ *($[a-zA-Z]+)', r'\1\+\2', line)

    line = re.sub(
        r'\$([a-zA-Z]+)',
        lambda m: (
            'L_' + m.group(1).upper()
            if m.group(1) in lang_dict
            else '$' + m.group(1)
        ),
        line,
    )
    line = re.sub(r'\[\"\.\((L_[A-Z_+]+)\)\.\"\]', r'[\1]', line)

    line = re.sub(
        'L_([A-Z]+)', lambda m: str(lang_dict[m.group(1).lower()]), line
    )
    for _ in range(4):
        line = re.sub(
            r'([0-9]+) *\+ *([0-9]+)',
            lambda m: str(int(m.group(1)) + int(m.group(2))),
            line,
        )

    if fn == 'lang':
        if len(line.split(',')) >= 3:
            parts = line.split(',')
            parts[0] = re.sub('/(.+?)/', r'\1', parts[0])
            # parts[1] = re.sub('\$', 'L_', parts[1])
            # parts[1] = re.sub(' *\+ *', '|', parts[1])
            parts[2] = parts[2].title()
            line = ','.join(parts)

    if 'languagenames' in fn:
        line = line.replace('"', "'")
        line = line.replace("','", "', '")
        if line and line[0] == "'":
            line = ' ' * 14 + line

    # fix upstream
    # line = line.replace('ë', 'ü')

    comment = ''
    if '#' in line:
        hashsign = line.find('#')
        comment = line[hashsign:]
        code = line[:hashsign]
    else:
        code = line

    code = code.rstrip()
    comment = comment.strip()
    if not re.match(r'^\s*$', code):
        comment = '  ' + comment

    if '(' in code and ')' in code:
        prefix = code[: code.find('(') + 1]
        suffix = code[code.rfind(')') :]
        tuplecontent = code[len(prefix) : len(code) - len(suffix)]

        elts = tuplecontent.split(',')
        for i in range(len(elts)):
            elts[i] = elts[i].strip()
            if elts[i][0] == '"' and elts[i][-1] == '"':
                elts[i] = "'" + elts[i][1:-1].replace("'", "\\'") + "'"
        tuplecontent = ', '.join(elts)

        code = prefix + tuplecontent + suffix

    line = code + comment
    line = re.sub('# *', '# ', line)

    if line:
        nl = False
        if array_seen and not (line[0] == '_' or line.startswith('BMDATA')):
            line = ' ' * 4 + line
        return line + '\n'
    elif not nl:
        nl = True
        return '\n'
    else:
        return ''


def _run_script():
    global array_seen, nl, sd, tail_text

    if len(sys.argv) > 1:
        bmdir = sys.argv[1].rstrip('/') + '/'
    else:
        bmdir = '../../bmpm/'

    outfilename = '../abydos/phonetic/_beider_morse_data.py'
    outfile = codecs.open(outfilename, 'w', 'utf-8')

    outfile.write(
        '# Copyright 2014-2020 by \
Christopher C. Little.\n# This file is part of Abydos.\n#\n# This file is \
based on Alexander Beider and Stephen P. Morse\'s implementation\n# of the \
Beider-Morse Phonetic Matching (BMPM) System, available at\n# \
http://stevemorse.org/phonetics/bmpm.htm.\n#\n# Abydos is free software: \
you can redistribute it and/or modify\n# it under the terms of the GNU \
General Public License as published by\n# the Free Software Foundation, \
either version 3 of the License, or\n# (at your option) any later version.\n\
#\n# Abydos is distributed in the hope that it will be useful,\n# but WITHOUT \
ANY WARRANTY; without even the implied warranty of\n# MERCHANTABILITY or \
FITNESS FOR A PARTICULAR PURPOSE. See the\n# GNU General Public License for \
more details.\n#\n# You should have received a copy of the GNU General Public \
License\n# along with Abydos. If not, see <http://www.gnu.org/licenses/>.\n\n\
"""abydos.phonetic._beider_morse_data.\n\nBehind-the-scenes constants, \
rules, etc. for the Beider-Morse Phonentic\nMatching (BMPM) algorithm\n\nDO \
NOT EDIT - This document is automatically generated from the reference\n\
implementation in PHP.\n"""\n\nfrom \
__future__ import (\n    absolute_import,\n    division,\n    print_function,\
    unicode_literals,\n)\n'
    )

    outfile.write('L_NONE = 0\n')
    for i, l in enumerate(lang_tuple):
        outfile.write('L_' + l.upper() + ' = 2**' + str(i) + '\n')
    outfile.write('\n\n')

    tail_text += '\nBMDATA = {}  # type: ignore\n'

    subdirs = ('gen', 'sep', 'ash')

    for s in subdirs:
        sd = s
        tail_text += "\nBMDATA['" + s + "'] = {}\n"
        tail_text += "BMDATA['" + s + "']['approx'] = {}\n"
        tail_text += "BMDATA['" + s + "']['exact'] = {}\n"
        tail_text += "BMDATA['" + s + "']['rules'] = {}\n"
        tail_text += "BMDATA['" + s + "']['hebrew'] = {}\n\n"
        tail_text += (
            "BMDATA['"
            + s
            + "']['language_rules'] = _"
            + s.upper()
            + '_LANGUAGE_RULES\n'
        )
        tail_text += (
            "BMDATA['" + s + "']['languages'] = _" + s.upper() + '_LANGUAGES\n'
        )

        phps = [
            f
            for f in sorted(listdir(bmdir + s + '/'))
            if (isfile(bmdir + s + '/' + f) and f.endswith('.php'))
        ]
        for infilename in phps:
            for pfx in (
                'rules',
                'approx',
                'exact',
                'hebrew',
                'language',
                'lang',
            ):
                if infilename.startswith(pfx):
                    array_seen = False
                    infilepath = bmdir + s + '/' + infilename
                    infileenc = chardet.detect(open(infilepath, 'rb').read())[
                        'encoding'
                    ]
                    print(s + '/' + infilename)  # noqa: T001
                    infile = codecs.open(infilepath, 'r', infileenc)
                    # if infilename.startswith('lang'):
                    #     tuplename = infilename[:-4]
                    # else:
                    #     tuplename = pfx + '_' + infilename[len(pfx) : -4]
                    # indent = len(tuplename) + 21

                    outfile.write('# ' + s + '/' + infilename + '\n')

                    ignore = True
                    for line in infile:
                        if 'function Language' in line:
                            break
                        if not ignore:
                            if re.search(r'\?>', line):
                                ignore = True
                            else:
                                line = pythonize(line, infilename[:-4], s)
                                if line.startswith('BMDATA'):
                                    tail_text += line
                                else:
                                    outfile.write(line)
                        if '*/' in line:
                            ignore = False

                    outfile.write('\n\n')
                    break

    outfile.write(tail_text)

    outfile.close()
    outfilelines = codecs.open(outfilename, 'r', 'utf-8').readlines()
    outfile = codecs.open(outfilename, 'w', 'utf-8')
    nl = False
    fixlanguagesarray = False

    sep_lang = (
        "('any', 'french', 'hebrew', 'italian', 'portuguese', 'spanish')"
    )

    for line in outfilelines:
        line = line.rstrip()
        if line:
            if fixlanguagesarray:
                line = ' ' + line.strip()
                fixlanguagesarray = False
            if len(line) > 79 or sep_lang in line:
                line += '  # noqa: E501'
            outfile.write(line)
            if not line.endswith('='):
                outfile.write('\n')
            else:
                fixlanguagesarray = True
            nl = False
        else:
            if not nl:
                outfile.write('\n')
            nl = True

    outfile.write(
        "\n\nif __name__ == '__main__':\n    import doctest\n\n\
    doctest.testmod()\n"
    )


if __name__ == '__main__':
    _run_script()
